{**********************************************************}
{                                                          }
{   TMaskEditHV 5.0                                        }
{   Autor: Hamden Vogel                                    }
{   E-mail: hamdenvogel@gmail.com                          }
{   http://www.hvogel.com.br                               }
{                                                          }
{   Freeware. ltima atualizao: 16/03/2012               }
{                                                          }
{**********************************************************}
{                                                          }
{**********************************************************}
{                                                          }
{ Componente descendente do TMaskedit com vrios           }
{ tipos de formatao e validao, dependendo da           }
{ mscara escolhida.                                       }
{  possvel tambm estender os tipos de formatao.       }
{ Tambm suporta outras funcionalidades, como alterao da }
{ da cor quando est com o foco e processamento automtico }
{ quando pressionada a tecla <Enter>.                      }
{                                                          }
{ Formatao e Validao de:                               }
{ * Data                                                   }
{ * Data/Hora                                              }
{ * Hora                                                   }
{ * CPF                                                    }
{ * CNPJ                                                   }
{ * CEP                                                    }
{ * Email                                                  }
{ * Profisso                                              }
{ * Escolaridade                                           }
{ * Moeda                                                  }
{ * Telefone                                               }
{ * Usurio (Nome)                                         }
{ * Endereo                                               }
{ * Estado                                                 }
{ * Cidade                                                 }
{ * Complemento                                            }
{ * Setor                                                  }
{ * Formato Customizvel                                   }
{                                                          }
{ Pode-se customizar:                                      }
{ 1: Mensagens de Erro (retornadas ao Usurio):            }
{   1.1: Formatar o Contedo da Mensagem;                  }
{                                                          }
{     1.1.1: Formatar o Nome do Componente associado a Mensagem; }
{                                                          }
{   1.2: Formatar o nome do Ttulo da Mensagem.            }
{                                                          }
{ 2: Opes de Validao:                                  }
{                                                          }
{   2.1: Se realiza a Validao ou no, de acordo com o preenchimento }
{   do campo. Se sim, o usurio somente deixar o campo se ele possuir}
{   a informao correta de acordo com o tipo de dado escolhido.      }
{                                                                     }
{     2.1.1: No caso de validao falsa, e se configurado para        }
{     criticar sua validao:                                         }
{                                                                     }
{       2.1.1.1: O componente exibir uma mensagem personalizada criticando sua validao incorreta; }
{     2.1.2: No caso de validao falsa, e se no configurado para    }
{     criticar sua validao:                                         }
{       2.1.2.1: O componente no exibir mensagem, mas prender o foco nele (crtica silenciosa).   }
{   2.2: Verificar se o campo est diferente de vazio:                }
{                                                                     }
{     2.2.1: Se realiza a verificao do campo preenchido ou no.     }
{     Se sim, o usurio somente deixar o campo se o mesmo estiver    }
{     preenchido. Note-se que a validao do contedo ser feita      }
{     de acordo com as configuraes definidas no item 1.             }
{                                                                     }
{      2.2.2: No caso de campo no preenchido, e se configurado para  }
{      criticar seu preenchimento (de acordo com as configuraes definidas }
{      no item 2.2.1):
{                                                                     }
{        2.2.2.1: O componente exibir uma mensagem personalizada criticando seu preenchimento; }
{      2.2.3: No caso de campo no preenchido, e se configurado para no }
{      criticar seu preenchimento (de acordo com as configuraes definidas }
{      no item 2.2.1):
{        2.2.3.1: O componente no exibir mensagem, mas prender o foco nele (crtica silenciosa). }
{
{ Portanto, as validaes so para assegurar que os campos esto      }
{ sendo preenchidos corretamente, tanto para o contedo em si, tanto  }
{ para preenchimento completo da informao.                          }
{  possvel customizar tipos de formatos,                  }
{ tipos de mensagens de erro,                               }
{ nome do campo, validao dos formatos,                    }
{ exibio (sim ou no) de mensagens ao usurio notificando }
{ campo vazio/invlido.                                     }


unit MaskEditHV;


interface


uses
  Mask, SysUtils, Math, Classes, Windows, Forms, Messages, StdCtrls,
  MaskUtils, DateUtils, Dialogs, Graphics, DB, DBCtrls, DBClient, ComCtrls, Typinfo;


const
 cVersion = '5.0';
 bNoFocus: Boolean = True;


type TMskFormate = (mskftDateTime, mskftDate, mskftTime, mskftCEP, mskftMoney, mskftTel, mskftCpf, mskftCnpj, mskftAddress,
                   mskftCity, mskftState, mskftSector, mskftUserName, mskftComplemento, mskftProfissao,
                   mskftEscolaridade, mskftEmail, mskftCustom);


type
  TSearchComboBox = class(TComboBox)
  private
    { Private declarations }
    FDelBksp : Boolean;
  protected
    { Protected declarations }
    procedure Change; override;
    procedure KeyPress(var Key: Char); override;
  //  procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
    { Public declarations }
    procedure AdicionaItem(Item : String);
  published
    { Published declarations }
  end;



type
  TMaskEditHV = class(TMaskEdit)
  private
     FFormate:TMskFormate;
     FMaskState:TMaskEditHV;
     FDataLink: TFieldDataLink;
     fInvalidMSG: string;
     fShowInvalidMSG: Boolean;
     fCheckEmptyField: Boolean;
     fNameField: String;
     fShowF1Date: Boolean;
     fCheckTime: Boolean;
     fCheckCPF: Boolean;
     fCheckCNPJ: Boolean;
     fCheckCEP: Boolean;
     fTelDDDFormat: Boolean;
     fCheckUserName: Boolean;
     fCheckAddress: Boolean;
     fCheckSector: Boolean;
     fCheckCity: Boolean;
     fCheckState: Boolean;
     fEscapeKeyPressed: Boolean;
     combo: TComboBox;
     customCombo: TSearchComboBox;
     MonthCalendar: TMonthCalendar;
     fTextHelp: TStrings;
     fComboCustomText: TStrings;
     fColorTextHelp: TColor;
     fShowTextHelp: Boolean;
     MemoHelp: TMemo;
     fShowCombo: Boolean;
     fOldShowCombo: Boolean;
     fShowList: Boolean;
     fShowListState: Boolean;
     fShowListCity: Boolean;
     fShowListProfissao: Boolean;
     fShowListEscolaridade: Boolean;
     fShowListCustom: Boolean;
     fCanAddMoreItens: Boolean;
     fCheckEmail: Boolean;
     fCheckProfissao: Boolean;
     fCheckEscolaridade: Boolean;
     FEmailMessage: string;
     fCheckComplemento: Boolean;
     fCheckMoney: Boolean;
     fCheckCustom: Boolean;
     fTitleErrorMSG: String;
     fTitleWarningMSG: String;
     fTitleOKMSG: String;
     //fEstado: string;
     fShowCurrencyString: Boolean;
     //fEstadoCEPAutoDetect: Boolean;
     fCannotContainEmptyCaracter: string;
     bSilentError: Boolean;
     fVersion: string;
     fCorAtiva: TColor;
     fCorNaoAtiva: TColor;

     procedure MyMessageBox;
     procedure SetInvalidMSG(const Value: string);
     procedure SetCheckEmptyField(const Value: Boolean);
     procedure SetNameField(const Value: string);
     function  GetFormate: TMskFormate;
     procedure DataChange(Sender: TObject);
     procedure UpdateData(Sender: TObject);
     procedure SetDataSource(Value: TDataSource);
     function  GetDataSource: TDataSource;
     procedure SetDataField(const value : string);
     function  GetDataField : string;
     function GetReadOnly: boolean;
     procedure SetReadOnly(const Value: boolean);
     // field linked to the control
     function  GetField : TField;
     procedure SetTextHelp(const Value: TStrings);
     procedure SetComboCustomText(const Value: TStrings);
     procedure SetColorTextHelp(const Value: TColor);
     procedure SetShowTextHelp(const Value: Boolean);
     procedure SetTitleErrorMSG(const Value: String);
     procedure SetTitleWarningMSG(const Value: String);
     procedure SetTitleOKMSG(const Value: String);
     procedure GoTab;
     procedure GoEnter;
     procedure SetShowF1Date(const Value: Boolean);
     function  extractResFromPas(binresname: string; path: string): boolean;
     procedure GetList(const NoFocus: Boolean = false; const bNoVisible: Boolean = False);
     procedure GetCustomList(const fKey: char = ' '; const NoFocus: Boolean = false);
     procedure GetCustomListNoVisible(const fKey: char = ' '; const NoFocus: Boolean = false);
     procedure GetMonthCalendar;
     procedure GetTextHelp;
     procedure MonthCalendarDblClick(Sender: TObject);
     procedure MonthCalendarKeyPress(Sender: TObject; var Key: Char);
     procedure MyChange(Sender: TObject);
     procedure MyKeyPress(Sender: TObject; var Key: Char);
     procedure MyExit(Sender: TObject);
     procedure CustomKeyPress(Sender: TObject; var Key: Char);
     procedure CustomExit(Sender: TObject);
     procedure SetShowCombo(const Value: Boolean);
     procedure SetListState(const Value: Boolean);
     procedure SetListCity(const Value: Boolean);
     procedure SetListProfissao(const Value: Boolean);
     procedure SetListEscolaridade(const Value: Boolean);
     procedure SetListCustom(const Value: Boolean);
     procedure SetCanAddMoreItens(const Value: Boolean);
     //procedure SetEstado(const Value: string);
     procedure SetShowCurrencyString(const Value: Boolean);
     //procedure SetEstadoCEPAutoDetect(const Value: Boolean);
     procedure SetVersion(const Value: String);
     procedure SetCannotContainEmptyCaracter(const Value: String);
     function HourExistsToo: Boolean;
     function WithoutAllSpaces(const S: String): String;
     //no utilizadas ainda:
     function FilterChar(const T : string) : string;
     // ****************
     function EmptyHourExistsInDateTimeFormat: Boolean;
     function MyFormatDate: string;

     function IsValidDate(const FStringDate: string): Boolean; overload;
     function IsValidTime(const FStringTime: string): Boolean; overload;
     function IsValidCPF(const FStringCPF: string): Boolean; overload;
     function IsValidCNPJ(const FStringCNPJ: string): Boolean; overload;
     procedure TrimMask;
     procedure UnTrimMask;
     procedure SetFormate(const Value: TMskFormate);
     procedure SetShowInvalidMSG(const Value: Boolean);
     procedure SetTelDDDFormat(const Value: Boolean);
     procedure SetFMaskState(const Value: TMaskEditHV);

  protected
     procedure KeyDown(var Key: Word; Shift: TShiftState); override;
     procedure KeyPress(var Key: Char); override;
     procedure Click; override;
     procedure DoEnter; override;
     procedure DoExit; override;
     procedure Change; override;
     procedure Notification(Acomponent:TComponent;operation:TOperation);override;
  public
     constructor Create (AOwner: TComponent); override;
     destructor  Destroy; override;

     procedure Teste;
     function EmptyDate: Boolean;
     function EmptyDateTime: Boolean;
     function EmptyTime: Boolean;
     function EmptyCPF: Boolean;
     function EmptyCNPJ: Boolean;
     function EmptyCEP: Boolean;
     function EmptyTel: Boolean;
     function EmptyTel1: Boolean;
     function EmptyTel2: Boolean;
     function EmptyText: Boolean;
     function EmptyMoney: Boolean;
     function EmptyExists: Boolean;
     function DateFormat: Boolean;
     function TimeFormat: Boolean;
     function DateTimeFormat: Boolean;
     function CPFFormat: Boolean;
     function CNPJFormat: Boolean;
     function CEPFormat: Boolean;
     function TelFormat1: Boolean;
     function TelFormat2: Boolean;
     function IsValidDate: Boolean; overload;
     function IsValidTime: Boolean; overload;
     function IsValidDateAndTime: Boolean;
     function IsValidCPF: Boolean; overload;
     function IsValidCNPJ: Boolean; overload;
     function IsValidCEP: Boolean;
     function IsValidEmail: Boolean;

     function   IsSilentError: Boolean;
     procedure  LoadTempXML(const tempXML: string; const tempIndex: string; const iObject: string);
     function   GetIndex: integer;
     procedure  LoadValueByIndex(const Value: string; const bNoVisible: Boolean = False);
     function   ValueExistsFromCombo(const Value: string): Boolean;
     Procedure  DelFile(vMasc:String);
     function   GetCurrentState: string; //obter o estado (UF) informado

  published

     property InvalidMSG: string read fInvalidMSG write SetInvalidMSG;
     property MaskState:TMaskEditHV read FMaskState  Write SetFMaskState;
     property Field : TField read getField;
     property DataField  : string read getDataField  write setDataField;
     property DataSource : TDataSource read getDataSource write setDataSource;
     property ShowInvalidMSG: Boolean read fShowInvalidMSG write SetShowInvalidMSG default true;
     property CheckEmptyField: Boolean  read fCheckEmptyField write SetCheckEmptyField default true;
     property NameField: string  read fNameField write SetNameField;
     property ShowCombo: Boolean read fShowCombo write SetShowCombo;
     property CanAddMoreItens: Boolean read fCanAddMoreItens write SetCanAddMoreItens default false;
     property Formate:TMskFormate read FFormate write SetFormate;
     property StringCannotContainEmptyCaracter: String read fCannotContainEmptyCaracter write SetCannotContainEmptyCaracter;
     property TitleErrorMSG: String read fTitleErrorMSG write SetTitleErrorMSG;
     property TitleWarningMSG: String read fTitleWarningMSG write SetTitleWarningMSG;
     property TitleOKMSG: String read fTitleOKMSG write SetTitleOKMSG;
    // property Estado: string read fEstado write SetEstado; //Estado do CEP ...
     property ShowCurrencyString: Boolean read fShowCurrencyString write SetShowCurrencyString;
     //property EstadoCEPAutoDetect: Boolean read fEstadoCEPAutoDetect write SetEstadoCEPAutoDetect;
     property TextHelp: TStrings read fTextHelp write SetTextHelp;
     property TextCustomCombo: TStrings read fComboCustomText write SetComboCustomText;
     property TextColorHelp: TColor read fColorTextHelp write SetColorTextHelp;
     property ShowTextHelp: Boolean read fShowTextHelp write SetShowTextHelp;
     property ShowF1Date: Boolean read fShowF1Date write SetShowF1Date;
     property ShowTelDDDFormat: Boolean read fTelDDDFormat write SetTelDDDFormat;
     property Version: string              read fVersion             write SetVersion;
     property CorAtiva: TColor             read fCorAtiva            write fCorAtiva;
     property CorNaoAtiva: TColor          read fCorNaoAtiva         write fCorNaoAtiva;
  end;


procedure Register;

implementation
{$R estados.res}
{$R cidades.res}
{$R escolaridade.res}
{$R profissao.res}


uses Controls, Types;

{ TSearchComboBox }


procedure TSearchComboBox.AdicionaItem(Item: String);
var
  ItemAtu : Integer;
begin
  ItemAtu := Items.IndexOf(Item);
  if ItemAtu < 0 then
// se o item no foi encontrado, insere
    Items.Insert(0,Item)
  else
// se item j existe, move para a primeira posio
    Items.Move(ItemAtu,0);
end;


procedure TSearchComboBox.Change;
var
  i, PosAtu : Integer;
  Tamanho : Integer;
begin
  inherited;
  if (Style in [csDropDown,csSimple]) and not FDelBksp then begin
// lembra posio atual
    PosAtu := SelStart;
// pesquisa se item existe na Combobox
    Tamanho := Length(Text);
    for i := 0 to Pred(Items.Count) do
      if AnsiCompareText(Text,Copy(Items[i],1,Tamanho)) = 0 then begin
// se encontrou atualiza o texto
        Text := Items[i];
        break;
      end;
// set the selection from the original position to the end of the text
    SelStart := PosAtu;
    SelLength := Length(Text);
  end;
  FDelBkSp := False;
end;


procedure TSearchComboBox.KeyPress(var Key: Char);
begin
  inherited;
  Key := UpCase(Key);
end;


procedure TMaskEditHV.GoTab;
begin
 // funo desabilitada, pois no  necessria:
 // a funo de tabulao j  suportada pelo Windows,
 // e tambm pelos componentes do Delphi.
end;


procedure TMaskEditHV.GoEnter;
begin
  // simular a tecla <Tab> quando pressionada a tecla <Enter>.
  PostMessage(Handle, WM_KEYDOWN, VK_TAB, 0);
  if fCheckEmail then Self.DoExit;
end;


procedure TMaskEditHV.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if (key = VK_ESCAPE) then
  begin
    fOldShowCombo      := fShowCombo;
    fEscapeKeyPressed  := True;
    if Self.ShowCombo then fShowCombo := False;
    GoEnter;
    Abort;
  end;

  if (key = vk_F1) and (fShowF1Date) and (DateFormat or DateTimeFormat) then
  begin
    GetMonthCalendar;
    Abort;
  end;

  if (Key in [13]) then
  begin
    if not (fShowCombo) then
    begin
      GoEnter;
      Abort;
    end;

    if (not(fShowListState) and not(fShowListCity) and not(fShowListProfissao) and not(fShowListEscolaridade)
    and not(fShowList)) or (not (fShowListCustom) and (fShowListCustom)) then
    begin
      GoEnter;
      Abort;
    end;
  end
  else if (key = (VK_DELETE)) and (fCheckMoney) then
  begin
    if fShowCurrencyString then Self.Text := FormatFloat('R$ #,0.00',0)
    else Self.Text := FormatFloat('#,0.00',0);

    Self.Click;
  end;
end;

  
procedure TMaskEditHV.KeyPress(var Key: Char);
var
  Texto, Texto2: string;
  i: byte;
begin
  inherited;
  if fCheckEmail then Self.CharCase := ecNormal;
 
  if ((fCheckState) and (fShowList)) or
  ((fCheckCity)  and (fShowList)) or
  ((fCheckProfissao) and (fShowList)) or
  ((fCheckEscolaridade) and (fShowList))
  and (key = #13) then
  if fShowCombo then
    GetList;

  if (fCheckCustom) and (fShowList) {and (key = #13)} then
    if fShowCombo then
      GetCustomList(Key);

  if fCheckMoney then
  begin
    if (Key in ['0'..'9',Chr(VK_BACK)]) then
    begin
      // limita-se a 23 caracteres seno haver um erro na funo StrToInt64()
      if (key in ['0'..'9']) and (Length(Trim(TMaskEditHV(Self).Text))>23) then
        key := #0;

      Texto2 := '0';
      Texto := Trim(TMaskEditHV(Self).Text)+Key;
      for i := 1 to Length(Texto) do
        if Texto[i] in ['0'..'9'] then
          Texto2 := Texto2 + Texto[i];

      if key = chr(vk_back) then
        Delete(Texto2,Length(Texto2),1);

      // formato o texto que depois ser colocado no MaskEditHV
      if fShowCurrencyString then
        Texto2 := FormatFloat('R$ #,0.00',StrToInt64(Texto2)/100)
      else
        Texto2 := FormatFloat('#,0.00',StrToInt64(Texto2)/100);

      TMaskEditHV(Self).Text := Texto2;
      TMaskEditHV(Self).SelStart := Length(Texto2);
    end;

    Key := #0;
  end;
end;  


procedure TMaskEditHV.Click;
begin
  inherited;
  if not fCheckMoney then Exit;

  // retorno o cursor para o final do texto, caso tenha sido utilizado
  // o mouse para mover o cursor dentro do MaskEditHV
  TMaskEditHV(Self).SelStart := Length(TMaskEditHV(Self).Text);
end;


procedure TMaskEditHV.DoEnter;
begin
  inherited;
  Self.Color := fCorAtiva;
  if Assigned(fTextHelp) then
    if (fTextHelp.Count > 0) and (fShowTextHelp) then GetTextHelp;

  if Assigned(combo) then
    if combo.Visible then
      combo.SetFocus;

  if Assigned(customCombo) then
    if customCombo.Visible then
      customCombo.SetFocus
end;
      

procedure TMaskEditHV.DoExit;
procedure GoTabAndAbort;
begin
  GoTab;
  Abort;
end;
begin
  // if AllDisabledCheckFunctions then RestoreDefaultCheckFunctions;
  Inherited;
  Self.Color        := fCorNaoAtiva;

  if fEscapeKeyPressed then
  begin
    fShowCombo := fOldShowCombo;
    fEscapeKeyPressed := False;
    exit;
  end;

  if Assigned(MemoHelp) then
  begin
    MemoHelp.Visible := False;
  end;

  if (DateTimeFormat)  then
  begin
    if (fCheckEmptyField) then
    begin
      if (EmptyDateTime) then
      begin
        MyMessageBox;
        SetFocus;
        Exit;
      end;

      if (EmptyExists) or (not Self.IsValidDateAndTime) then
      begin
        SetFocus;
        Exit;
      end;

    GoTabAndAbort;
    Exit;
  end
  else //end (fCheckEmptyField = DateTime)
  begin
    if (bSilentError) and (IsSilentError) then
    begin
      SetFocus;
      Exit;
    end;

    if (EmptyDateTime) then GoTabAndAbort;
    if (EmptyExists) or (not Self.IsValidDateAndTime) then
    begin
      SetFocus;
      Exit;
    end;

      GoTabAndAbort;
      Exit;
    end;
  end; // end  DateTime format

  if (DateFormat) and (fCheckEmptyField) then
  begin
    if Assigned(MonthCalendar) then
    begin
      if (MonthCalendar.Visible) then
      begin
        Self.Text := DateToStr(MonthCalendar.Date);
        Self.Text := MyFormatDate;
        MonthCalendar.Visible := False;
        GoTabAndAbort;
        Exit;
      end;
    end;

    if (EmptyDate) then
    begin
      MyMessageBox;
      SetFocus;
      Exit;
    end;

    if (EmptyExists) then
    begin
      SetFocus;
      Exit;
    end;

    if (not Self.IsValidDate) then
    begin
      SetFocus;
      Exit;
    end;

    GoTabAndAbort;
    Exit;
  end
  else if (DateFormat) then
  begin
    if Assigned(MonthCalendar) then
    begin
      if (MonthCalendar.Visible) then
      begin
        Self.Text := DateToStr(MonthCalendar.Date);
        Self.Text := MyFormatDate;
        MonthCalendar.Visible := False;
        GoTabAndAbort;
        Exit;
      end;
    end;

    if (bSilentError) and (IsSilentError) then
    begin
      SetFocus;
      Exit;
    end;

    if (EmptyExists) and not (EmptyDate) then
    begin
      SetFocus;
      Exit;
    end;

    if (EmptyDate) then GoTabAndAbort;
    if not (Self.IsValidDate) then
    begin
      SetFocus;
      Exit;
    end;

    GoTabAndAbort;
    Exit;
  end  // end  Date format
  else if (TimeFormat) then
  begin
    if (fCheckEmptyField) then
    begin
      if (EmptyTime) then
      begin
        MyMessageBox;
        SetFocus;
        Exit;
      end;

    if (EmptyExists) or (not Self.IsValidTime) then
    begin
      SetFocus;
      Exit;
    end;

    GoTabAndAbort;
  end //end  fCheckEmptyTime
  else
  begin
    if (bSilentError) and (IsSilentError) then
    begin
      SetFocus;
      Exit;
    end;

    if (EmptyTime) then GoTabAndAbort;

    if (EmptyExists) or (not Self.IsValidTime) then
    begin
      SetFocus;
      Exit;
    end;

      GoTabAndAbort;
    end;

    Exit;
  end // end  Time format
  else if (CPFFormat)  then
  begin
    if (fCheckEmptyField) then
    begin
      if (EmptyCPF) then
      begin
        MyMessageBox;
        SetFocus;
        Exit;
      end;

      if (EmptyExists) then
      begin
        SetFocus;
        Exit;
      end;
      
      if (not Self.IsValidCPF) then
      begin
        SetFocus;
        Exit;
      end;

    GoTabAndAbort;
    Exit;
  end
  else
  begin
    if (bSilentError) and (IsSilentError) then
    begin
      SetFocus;
      Exit;
    end;

    if (EmptyCPF) then GoTabAndAbort;
    if (EmptyExists) then
    begin
      SetFocus;
      Exit;
    end;

    if (not Self.IsValidCPF) then
    begin
      SetFocus;
      Exit;
    end;

     GoTabAndAbort;
     Exit;
   end;

    Exit;
  end // end  CPF format
  else if (CNPJFormat)  then
  begin
    if (fCheckEmptyField) then
    begin
      if (EmptyCNPJ) then
      begin
        MyMessageBox;
        SetFocus;
        Exit;
      end;

      if (EmptyExists) then
      begin
        SetFocus;
        Exit;
      end;

      if (not Self.IsValidCNPJ) then
      begin
        SetFocus;
        Exit;
      end;

     GoTabAndAbort;
     Exit;
    end
    else
    begin
      if (bSilentError) and (IsSilentError) then
      begin
        SetFocus;
        Exit;
      end;

      if (EmptyCNPJ) then GoTabAndAbort;
      if (EmptyExists) then
      begin
        SetFocus;
        Exit;
      end;

      if (not Self.IsValidCNPJ) then
      begin
        SetFocus;
        Exit;
      end;

      GoTabAndAbort;
      Exit;
    end;

    Exit;
  end  // end  CNPJ format
  else if (CEPFormat) then
  begin
    if (fCheckEmptyField) then
    begin
      if (EmptyCEP) then
      begin
        MyMessageBox;
        SetFocus;
        Exit;
      end;

      if (EmptyExists) then
      begin
        SetFocus;
        Exit;
      end;

      if (not Self.IsValidCEP) then
      begin
        SetFocus;
        Exit;
      end;

      GoTabAndAbort;
      Exit;
    end
    else
    begin
      if (bSilentError) and (IsSilentError) then
      begin
        SetFocus;
        Exit;
      end;

      if (EmptyCEP) then GoTabAndAbort;

      if (EmptyExists) then
      begin
        SetFocus;
        Exit;
      end;

      if (not Self.IsValidCEP) then
      begin
        SetFocus;
        Exit;
      end;

      GoTabAndAbort;
      Exit;
    end;
  end // end  CEP format
  else if (TelFormat1 or TelFormat2) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyTel) then
      begin
        MyMessageBox;
        SetFocus;
        Exit;
      end;

    if (EmptyExists) then
    begin
      SetFocus;
      Exit;
    end;

     {if (not Self.IsValidTel) then
     begin
       SetFocus;
       Exit;
     end;
     }
      GoTabAndAbort;
      Exit;
   end
   else
   begin
     if (bSilentError) and (IsSilentError) then
     begin
       SetFocus;
       Exit;
     end;

     if (EmptyTel) then GoTabAndAbort;
     if (EmptyExists) then
     begin
       SetFocus;
       Exit;
     end;

     {if (not Self.IsValidTel) then
     begin
       SetFocus;
       Exit;
     end;
     }
     GoTabAndAbort;
     Exit;
   end;

    Exit;
  end  // end  Tel format
  else if (fCheckEmail) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        MyMessageBox;
        SetFocus;
        Exit;
      end;

      if (not Self.IsValidEmail) then
      begin
        SetFocus;
        Exit;
      end;

      GoTabAndAbort;
      Exit;
    end
    else
    begin
      if (bSilentError) and (IsSilentError) then
      begin
        SetFocus;
        Exit;
      end;

      if (EmptyText) then GoTabAndAbort;

      if (not Self.IsValidEmail) then
      begin
        SetFocus;
        Exit;
      end;

      GoTabAndAbort;
      Exit;
    end;

    Exit;
  end  // end  fCheckEmail
  else if (fCheckProfissao) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
          MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end  // end  fCheckProfissao
  else if (fCheckEscolaridade) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
          MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end  // end  fCheckEscolaridade
  else if (fCheckUserName) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
          MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end  // end  fCheckUserName
  else if (fCheckAddress) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
          MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end  // end  fCheckAddress
  else if (fCheckSector) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
          MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end  // end  fCheckSector
  else if (fCheckState) then
  begin
   //fEstado := Self.Text;

    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
          MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end // end  fCheckState
  else if (fCheckCity) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
        MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end // end fCheckCity
  else if (fCheckComplemento) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
        MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end // end fCheckComplemento
  else if (fCheckMoney) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyMoney) then
      begin
        MyMessageBox;
        SetFocus;
        Exit;
      end;

      GoTabAndAbort;
      Exit;
    end;
  end  // end fCheckMoney
  else if (fCheckCustom) then
  begin
    if (CheckEmptyField) then
    begin
      if (EmptyText) then
      begin
        if (fShowInvalidMSG) then
        MyMessageBox;
        SetFocus;
        Exit;
      end;
    end;
    GoTabAndAbort;
    Exit;
  end // end fCheckCustom
 // inherited;
end;


procedure TMaskEditHV.Change;
begin
  if fDataLink.Field <> nil then
  begin
    if fDataLink.Edit then
    begin
      inherited Change;
      fDataLink.Modified;
    end;
  end;
end;


constructor TMaskEditHV.Create(AOwner: TComponent);
begin
 inherited;
//  if (csDesigning     in ComponentState) then SetFormate(mskftDateTime);
 with Self do
 begin
   fEscapeKeyPressed   := False;
   fNameField          := '[Nome deste Campo]';
   fInvalidMSG         := '%s Invlido!';
   fShowInvalidMSG     := True;
   fCheckEmptyField    := True;
   // Width               := 75;
   bSilentError        := False;
   CharCase            := ecUpperCase;
   fVersion            := cVersion;
   fCorAtiva           := clYellow;
   fCorNaoAtiva        := $00D8FEFC;
   Self.Color          := fCorNaoAtiva;

   FDataLink                 := TFieldDataLink.Create;
   FDataLink.Control         := Self;
   FDataLink.OnDataChange    := DataChange;
   FDataLink.OnUpdateData    := UpdateData;

   fCannotContainEmptyCaracter := 'O campo %s no est totalmente preenchido!';
   //fEstado               := 'GO'; //Estado de Gois, como default ...
  
   if (fCheckMoney) and (fShowCurrencyString) then
     Self.Text := FormatFloat('R$ #,0.00',0)
   else if (fCheckMoney) then
     Self.Text := FormatFloat('#,0.00',0);
   fTextHelp            := TStringList.Create;
   fComboCustomText     := TStringList.Create;
   fColorTextHelp       := clBlue;
   fShowTextHelp        := True; //default = true, mas s exibe se conter algum texto ..
   fShowF1Date          := False;
   //fEstadoCEPAutoDetect  := True;
   fTelDDDFormat       := False;
   fTitleErrorMSG      := 'Erro';
   fTitleWarningMSG    := 'Ateno';
   fTitleOKMSG         := 'OK';
 end;
end;

destructor TMaskEditHV.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;

  inherited destroy;
end;


function TMaskEditHV.EmptyDate: Boolean;
begin
  Result := (Self.Text = '  /  /    ');
end;

function TMaskEditHV.EmptyDateTime: Boolean;
begin
  Result := (Self.Text = '  /  /       :  ');
end;

function TMaskEditHV.EmptyTime: Boolean;
begin
  Result := (Self.Text = '  :  ');
end;

function TMaskEditHV.EmptyCPF: Boolean;
begin
  Result := (Self.Text = '   .   .   -  ');
end;

function TMaskEditHV.EmptyCNPJ: Boolean;
begin
  Result := (Self.Text = '  .   .   /    -  ');
end;

function TMaskEditHV.EmptyCEP: Boolean;
begin
  Result := (Self.Text = '  .   -   ');
end;

function TMaskEditHV.EmptyTel: Boolean;
begin
  if TelFormat1 then Result := EmptyTel1
  else if TelFormat2 then Result := EmptyTel2;
end;

function TMaskEditHV.EmptyTel1: Boolean;
begin
  Result := (Self.Text = '(  )     -    ');
end;
  
function TMaskEditHV.EmptyTel2: Boolean;
begin
  Result := (Self.Text = '(   )     -    ');
end;

function TMaskEditHV.EmptyMoney: Boolean;
var
  auxEmptyMoney: string;
begin
  if fShowCurrencyString then
    auxEmptyMoney := FormatFloat('R$ #,0.00',0)
  else
    auxEmptyMoney := FormatFloat('#,0.00',0);
  Result := (Self.Text = auxEmptyMoney) or (Self.Text = '');
end;


function TMaskEditHV.HourExistsToo: Boolean;
begin
  Result := (System.Pos(':', Copy(Self.Text,14,1)) = 1);
end;


function TMaskEditHV.IsValidDate(const FStringDate: string): Boolean;
begin
  Result := False;
  if (FStringDate = '') or (not DateFormat) then Exit;

  if HourExistsToo then          //formato: dd/mm/yyyy hh:mm
    Result := (DateUtils.IsValidTime( (StrToIntDef(System.Copy(FStringDate,12,2),0)), (StrToIntDef(System.Copy(FStringDate,15,2),0)),
    0, 0))
  and (DateUtils.IsValidDate( (StrToIntDef(System.Copy(FStringDate,7,4),0)), (StrToIntDef(System.Copy(FStringDate,4,2),0)),
    (StrToIntDef(Copy(FStringDate,1,2),0)))) and not (EmptyHourExistsInDateTimeFormat)
  else
    Result := DateUtils.IsValidDate( (StrToIntDef(System.Copy(FStringDate,7,4),0)), (StrToIntDef(System.Copy(FStringDate,4,2),0)),
    (StrToIntDef(Copy(FStringDate,1,2),0)));
  if (not Result) and (fShowInvalidMSG) and (DateTimeFormat) then
    Application.MessageBox(Pchar(Format(InvalidMSG,[NameField])), Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR)
  else if (not Result) and (fShowInvalidMSG) and (DateFormat) and (not DateTimeFormat)  then
    Application.MessageBox(Pchar(Format(InvalidMSG, [NameField])),  Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR)
end;


function TMaskEditHV.IsValidDate: Boolean;
begin
//  Result := (EmptyExists);
//  if not Result then
  Result := Self.IsValidDate(Self.Text) and (not EmptyDate) ;
end;
  

function TMaskEditHV.IsValidTime(const FStringTime: string): Boolean;
begin
  Result := False;  //hh:mm
  if (FStringTime = '') or (EmptyTime) or (EmptyExists) or (not TimeFormat) then Exit;
    Result := DateUtils.IsValidTime( (StrToIntDef(System.Copy(FStringTime,1,2),0)),
  (StrToIntDef(System.Copy(FStringTime,4,5),0)),0,0);

  if (not Result) and (fShowInvalidMSG) then
    MyMessageBox;
end;


function TMaskEditHV.IsValidTime: Boolean;
begin
  Result := Self.IsValidTime(Self.Text);
end;

function TMaskEditHV.IsValidDateAndTime: Boolean;
begin
  Result := Self.IsValidDate(Self.Text);
end;

function TMaskEditHV.IsValidCEP: Boolean;
var
  cCEP1, i: Integer;
  cCep: string;
  fEstado: string;
begin
  Result  := False;
  cCep    := Self.Text;
 {
 if EstadoCEPAutoDetect then //detecta automaticamente se existe um objeto j contendo o estado preenchido.
 begin                      //Se sim, j obtm seu valor e valida conforme o valor do CEP fornecido.
   for I := 0 to pred(Owner.ComponentCount) do
   begin
     if (Owner.Components[I] is TMaskeditHV) then
     begin
       if (TMaskeditHV(Owner.Components[I]).fCheckState) and (TMaskeditHV(Owner.Components[I]).Estado <> '') then
       begin
         fEstado :=  TMaskeditHV(Owner.Components[I]).Estado;
         Break;
       end;
     end;
   end;
 end;
 }

  fEstado := GetCurrentState;
  if (fEstado = '') then
  begin
    Application.MessageBox('Favor informar o Estado!',Pchar(TitleWarningMSG),MB_OK + MB_ICONWARNING);
    Exit;
  end;

  if (cCep = '') or (fEstado = '') or (EmptyCEP) or (EmptyExists) then Exit;

  cCep := SysUtils.StringReplace(cCep,'.','',[rfReplaceAll]);
  cCep := SysUtils.StringReplace(cCep,'-','',[rfReplaceAll]);
  cCep := WithoutAllSpaces(cCep);

  fEstado := SysUtils.StringReplace(fEstado,'.','',[rfReplaceAll]);
  fEstado := SysUtils.StringReplace(fEstado,'-','',[rfReplaceAll]);
  fEstado := WithoutAllSpaces(fEstado);
           
  cCep := copy(cCep,1,5) + copy(cCep,7,3);
  cCEP1 := StrToInt(copy(cCep,1,3));
  if Length(trim(cCep)) > 0 then
  begin
    if (StrToInt(cCep) <= 1000000.0) then
    begin
      if fShowInvalidMSG then
        Application.MessageBox('CEP tem que ser maior que [10000-000]',Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
      Result := False
    end
    else
    begin
      {if Length(trim(copy(cCep,6,3))) < 3 then Result := False else   }
      if (fEstado = 'SP') then
      begin
        Result := (cCEP1 >= 10 ) and (cCEP1 <= 199);
        if not Result then
        begin
          if fShowInvalidMSG then
            Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de SP, os trs nmeros iniciais do'#13#10 +
            'CEP devem estar entre 10 e 199.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
          SetFocus;
          Exit;
        end;
      end
      else
      if (fEstado = 'RJ') then
        begin
          Result := (cCEP1 >= 200) and (cCEP1 <= 289);
          if not Result then
          begin
            if fShowInvalidMSG then
              Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do RJ, os trs nmeros iniciais do'#13#10 +
              'CEP devem estar entre 200 e 289.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
            SetFocus;
            Exit;
          end;
        end
        else
        if (fEstado = 'ES') then
        begin
          Result := (cCEP1 >= 290) and (cCEP1 <= 299);
          if not Result then
          begin
            if fShowInvalidMSG then
              Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do ES, os trs nmeros iniciais do'#13#10 +
              'CEP devem estar entre 290 e 299.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
            SetFocus;
            Exit;
         end;
       end
       else
       if (fEstado = 'MG') then
       begin
         Result := (cCEP1 >= 300) and (cCEP1 <= 399);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de MG, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 300 e 399.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'BA') then
       begin
         Result := (cCEP1 >= 400) and (cCEP1 <= 489);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado da BA, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 400 e 489.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'SE') then
       begin
         Result := (cCEP1 >= 490) and (cCEP1 <= 499);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de SE, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 490 e 499.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'PE') then
       begin
         Result := (cCEP1 >= 500) and (cCEP1 <= 569);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de PE, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 500 e 569.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'AL') then
       begin
         Result := (cCEP1 >= 570) and (cCEP1 <= 579);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de AL, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 570 e 579.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'PB') then
       begin
         Result := (cCEP1 >= 580) and (cCEP1 <= 589);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado da PB, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 580 e 589.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'RN') then
       begin
         Result := (cCEP1 >= 590) and (cCEP1 <= 599);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do RN, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 590 e 599.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'CE') then
       begin
         Result := (cCEP1 >= 600) and (cCEP1 <= 639);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do CE, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 600 e 639.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'PI') then
       begin
         Result := (cCEP1 >= 640) and (cCEP1 <= 649);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do PI, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 640 e 649.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'MA') then
       begin
         Result := (cCEP1 >= 650) and (cCEP1 <= 659);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do MA, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 650 e 659.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'PA') then
       begin
         Result := (cCEP1 >= 660) and (cCEP1 <= 688);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do PA, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 660 e 688.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'AM') then
       begin
         Result := ((cCEP1 >= 690) and (cCEP1 <= 692) or (cCEP1 >= 694) and
         (cCEP1 <= 698));
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do AM, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 690 e 692 ou entre 694 e 698.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'AP') then
       begin
         Result := (cCEP1 = 689);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do AP, os trs nmeros iniciais do'#13#10 +
             'CEP devem ser 689.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'RR') then
       begin
         Result := (cCEP1 = 693);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de RR, os trs nmeros iniciais do'#13#10 +
             'CEP devem ser 693.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'AC') then
       begin
         Result := (cCEP1 = 699);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do AC, os trs nmeros iniciais do'#13#10 +
             'CEP devem ser 699.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if ((fEstado = 'DF') or (fEstado = 'GO')) then
       begin
         Result := (cCEP1 >= 000) and (cCEP1 <= 999);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do DF ou GO, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 000 e 999.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'TO') then
       begin
         Result := (cCEP1 >= 770) and (cCEP1 <= 779);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de TO, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 770 e 779.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'MT') then
       begin
         Result := (cCEP1 >= 780) and (cCEP1 <= 788);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do MT, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 780 e 788.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'MS') then
       begin
         Result := (cCEP1 >= 790) and (cCEP1 <= 799);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do MS, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 790 e 799.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'RO') then
       begin
         Result := (cCEP1 = 789);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de RO, os trs nmeros iniciais do'#13#10 +
             'CEP devem ser 789.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'PR') then
       begin
         Result := (cCEP1 >= 800) and (cCEP1 <= 879);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do PR, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 800 e 879.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'SC') then
       begin
         Result := (cCEP1 >= 880) and (cCEP1 <= 899);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado de SC, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 880 e 899.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end
       else
       if (fEstado = 'RS') then
       begin
         Result := (cCEP1 >= 900) and (cCEP1 <= 999);
         if not Result then
         begin
           if fShowInvalidMSG then
             Application.MessageBox(Pchar(Format(InvalidMSG, [NameField]) + #13#10'Erro: No Estado do RS, os trs nmeros iniciais do'#13#10 +
             'CEP devem estar entre 900 e 999.'),Pchar(TitleErrorMSG), MB_OK + MB_ICONERROR);
           SetFocus;
           Exit;
         end;
       end;
     end;
   end
   else Result := True;
end;

                                                                  {
function TMaskEditHV.IsValidTel(const fStringTel: string): Boolean;
function EmptyExistsTelFormat1: Boolean;
begin
 Self.EditMask := '(99)9999-9999;1;_';
 Result := EmptyExists;
 Self.EditMask := '(99) 9999-9999;1;_';
end;
function EmptyExistsTelFormat2: Boolean;
begin
 Self.EditMask := '(999)9999-9999;1;_';
 Result := EmptyExists;
 Self.EditMask := '(999) 9999-9999;1;_';
end;

begin
 Result := False;

 if (TelFormat1) then
 begin
   Result := (EmptyExistsTelFormat1);
   if not (Result) then
     if (fShowInvalidTelMSG) and (not EmptyTel) then
       Application.MessageBox(Pchar(Format(InvalidTelMSG,[TELName])),Pchar(TitleErrorMSG),MB_OK + MB_ICONERROR);
 end
 else if (TelFormat2) then
 begin
   Result := true;// (EmptyExistsTelFormat2);
   if not (Result) then
     if (fShowInvalidTelMSG) and (not EmptyTel) then
       Application.MessageBox(Pchar(Format(InvalidTelMSG,[TELName])),Pchar(TitleErrorMSG),MB_OK + MB_ICONERROR);
 end;
end;

function TMaskEditHV.IsValidTel: Boolean;
begin
 Result := (Self.IsValidTel(Self.Text));
end;
}


function TMaskEditHV.IsValidEmail: Boolean;
const
  vet_valido: array [0..35] of string = ('0','1','2','3','4','5','6','7',
  '8','9','a','b','c','d','e','f', 'g','h','i','j','k','l','m','n', 'o',
  'p','q','r','s','t','u','v', 'w','x','y','z');
const
  msg1 = 'Caractere(s) invlido(s) no incio do e-mail.';
  msg2 = 'Smbolo @ no foi encontrado.';
  msg3 = 'Excesso do smbolo @.';
  msg4 = 'Caractere(s) invlido(s) antes do smbolo @.';
  msg5 = 'Caractere(s) invlido(s) depois do smbolo @.';
  msg6 = 'Agrupamento de caractere(s) invlido(s) a esqueda do @.';
  msg7 = 'No existe ponto(s) digitado(s).';
  msg8 = 'Ponto encontrado no final do e-mail.';
  msg9 = 'Ausncia de caractere(s) aps o ltimo ponto.';
  msg10 = 'Excesso de ponto(s) a direita do @.';
  msg11 = 'Ponto(s) disposto(s) de forma errada aps o @.';
  msg12 = 'Caractere(s) invlido(s) antes do ponto.';
  msg13 = 'Caractere(s) invlido(s) depois do ponto.';
var
  i, j, tam_email, simb_arroba, simb_arroba2, qtd_arroba, qtd_pontos,
  qtd_pontos_esq, qtd_pontos_dir, posicao, posicao2, ponto, ponto2: integer;
  vet_email: array [0..49] of string; //50 posies, capacidade do Edit
  email, msg: string;
begin
  FEmailMessage := ''; //essa inicializao significa que enquanto varivel vazia est tudo OK ...
  qtd_pontos:= 0; qtd_pontos_esq:= 0; qtd_pontos_dir:= 0; qtd_arroba:= 0;
  posicao:=0; posicao2:=0; simb_arroba:=0; simb_arroba2:=0; ponto:= 0;
  ponto2:= 0; msg:='';
  Result:= True;
  email := Self.Text;
  //Verificando parte inicial do E-mail
  tam_email:= Length(email);
  for i:= 0 to tam_email-1 do
  begin
    vet_email[i]:= Copy(email,i+1,1);
    if vet_email[i] = '@' then
    begin
      Inc(qtd_arroba);
      posicao:= i;
    end;
  end;
  if ((vet_email[0] = '@') or (vet_email[0] = '.') or (vet_email[0] = '-')) then
  begin
    Result:= False;
    msg:= msg1;
  end;
  //Verificando se tem o smbolo @ e quantos tem
  if qtd_arroba < 1 then
  begin
    Result:= False;
    msg:= msg2;
  end
  else if qtd_arroba > 1 then
  begin
    Result:= False;
    msg:= msg3 + ' Encontrado(s): '+IntToStr(qtd_arroba)+'.';
  end
  else
  //Verificando o que vem antes e depois do smbolo @
  begin
  for i:=0 to 35 do
  begin
    if vet_email[posicao-1] <> vet_valido[i] then Inc(simb_arroba)
    else Dec(simb_arroba);
    if vet_email[posicao+1] <> vet_valido[i] then Inc(simb_arroba2)
    else Dec(simb_arroba2);
  end;
  if simb_arroba = 36 then
  begin
   //Antes do arroba h um smbolo desconhecido do vetor vlido
   Result:= False;
   msg:= msg4;
  end
  else if simb_arroba2 = 36 then
  begin
   //Depois do arroba h um smbolo desconhecido do vetor vlido
    Result:= False;
    msg:= msg5;
  end
  end;
  //Verificando se h pontos e quantos, e Verificando parte final do e-mail
  for j:=0 to tam_email-1 do
  if vet_email[j] = '-' then
  if ((vet_email[j-1] = '.') or (vet_email[j-1] = '-')) then
  begin
    Result:= False;
    msg:= msg6;
  end;
  for i:=0 to tam_email-1 do
  if vet_email[i] = '.' then
  begin
    Inc(qtd_pontos);
    posicao2:= i+1;
    if i > posicao then Inc(qtd_pontos_dir)
    else Inc(qtd_pontos_esq);
    if ((vet_email[i-1] = '.') or (vet_email[i-1] = '-')) then
    begin
      Result:= False;
      msg:= msg6;
    end;
  end;
  if qtd_pontos < 1 then
  begin
    Result:= False;
    msg:= msg7;
  end
  else if vet_email[tam_email-1] = '.' then
  begin
    Result:= False;
    msg:= msg8;
  end
  else if vet_email[tam_email-2] = '.' then
   begin
    Result:= False;
    msg:= msg9;
  end
  else if qtd_pontos_dir > 2 then
  begin
    Result:= False;
    msg:= msg10 + ' Encontrado(s): '+
    IntToStr(qtd_pontos)+#10+'Encontrado(s) a direita do @: '+
    IntToStr(qtd_pontos_dir)+'.';
  end
  else if (not ((((tam_email - posicao2) = 3) and (qtd_pontos_dir = 1)) or
  (((tam_email - posicao2) = 2) and (qtd_pontos_dir = 2)) or
  (((tam_email - posicao2) = 2) and (qtd_pontos_dir = 1)))) then
  begin
    Result:= False;
    msg:= msg11 +#10+ 'Encontrado(s) a esquerda do @: '+
    IntToStr(qtd_pontos_esq) +#10+ 'Encontrado(s) a direita do @: '+
    IntToStr(qtd_pontos_dir)+'.';
  end
  else
  //Verificando o que vem antes e depois do ponto
  begin
    for i:=0 to 35 do
    begin
      if vet_email[posicao2-2] <> LowerCase(vet_valido[i]) then Inc(ponto)
      else Dec(ponto);
     if vet_email[posicao2] <> LowerCase(vet_valido[i]) then Inc(ponto2)
      else Dec(ponto2);
    end;
    if ponto = 36 then
    begin
      //Antes do ponto h um smbolo desconhecido do vetor vlido
      Result:= False;
      msg:= msg12;
    end
    else if ponto2 = 36 then
    begin
      //Depois do ponto h um smbolo desconhecido do vetor vlido
      Result:= False;
      msg:= msg13;
    end
  end;
  //Verificao final
  if not Result then
  begin
    msg:= msg +#10+ 'Formato de E-mail no aceitvel!!';
    FEmailMessage := msg;
    fInvalidMSG := FEmailMessage;
    MyMessageBox;
   // Application.MessageBox(Pchar(msg)
  //  MessageDlg(msg,mtWarning,[mbRetry],0);
  end;
end;
    

function TMaskEditHV.IsValidCPF(const FStringCPF: string): Boolean;
function ValidCPF(const num: string): boolean;
var
  n1,n2,n3,n4,n5,n6,n7,n8,n9: integer;
  d1,d2: integer;
  digitado, calculado: string;
begin
  n1:=StrToInt(num[1]);
  n2:=StrToInt(num[2]);
  n3:=StrToInt(num[3]);
  n4:=StrToInt(num[4]);
  n5:=StrToInt(num[5]);
  n6:=StrToInt(num[6]);
  n7:=StrToInt(num[7]);
  n8:=StrToInt(num[8]);
  n9:=StrToInt(num[9]);
  d1:=n9*2+n8*3+n7*4+n6*5+n5*6+n4*7+n3*8+n2*9+n1*10;
  d1:=11-(d1 mod 11);
  if d1>=10 then
    d1:=0;
  d2:=d1*2+n9*3+n8*4+n7*5+n6*6+n5*7+n4*8+n3*9+n2*10+n1*11;
  d2:=11-(d2 mod 11);

  if d2>=10 then d2:=0;
    calculado:=inttostr(d1)+inttostr(d2);
  digitado:=num[10]+num[11];
  Result := (calculado=digitado);
end;

var
  auxStr: String;
begin
  Result := False;
  if (FStringCPF = '') or (EmptyCPF) or (EmptyExists) then Exit;

  auxStr := SysUtils.StringReplace(FStringCPF,'.','',[rfReplaceAll]);
  auxStr := SysUtils.StringReplace(auxStr,'-','',[rfReplaceAll]);
  auxStr := WithoutAllSpaces(auxStr);
  Result := ValidCPF(auxStr);
  if (not Result) and (fShowInvalidMSG) then
    MyMessageBox;
end;
   

function TMaskEditHV.IsValidCPF: Boolean;
begin
  Result := Self.IsValidCPF(Self.Text);
end;
 

function TMaskEditHV.IsValidCNPJ(const FStringCNPJ: string): Boolean;
var
  dig1,dig2,i,cal : integer;
  digitos : array [1..14] of integer;
  controle : boolean;
begin
  controle := false;
  if not((pos(' ',FStringCNPJ)<>0) or (FStringCNPJ = '')) then
  begin
    for i := 1 to length(FStringCNPJ) - 4 do // Copia string no vetor
    begin
      if i < 3 then
        digitos[i] := StrToInt(FStringCNPJ[i])
      else if (i >= 3) and (i < 6) then
        digitos[i] := StrToInt(FStringCNPJ[i+1])
      else if (i >= 6) and (i < 9) then
        digitos[i] := StrToInt(FStringCNPJ[i+2])
      else if (i >= 9) and (i < 13) then
        digitos[i] := StrToInt(FStringCNPJ[i+3])
      else if (i >= 13) then
        digitos[i] := StrToInt(FStringCNPJ[i+4]);
    end;
 // Testa c  --> 00.000.000.000/0000-00
  for i := 1 to 13 do
  if not((digitos[i] = 0) and (digitos[i+1] = 0)) then
  begin
    controle := true;
    break;
  end;

  if controle then // primeiro digito
  begin
    cal := (5 * digitos[1])+(4 * digitos[2])+(3 * digitos[3])+(2 * digitos[4])+
    (9 * digitos[5])+(8 * digitos[6])+(7 * digitos[7])+(6 * digitos[8])+
    (5 * digitos[9])+(4 * digitos[10])+(3 * digitos[11])+(2 * digitos[12]);

  if ((cal mod 11) = 1)or((cal mod 11) = 0) then
    dig1 := 0
  else
    dig1 := 11-(cal mod 11);

  if dig1 = digitos[13] then // Compara o 1 se ok vai calcular o 2 digito
  begin
    cal :=(6 * digitos[1])+(5 * digitos[2])+(4 * digitos[3])+(3 * digitos[4])+
    (2 * digitos[5])+(9 * digitos[6])+(8 * digitos[7])+(7 * digitos[8])+(6 * digitos[9])+
    (5 * digitos[10])+(4 * digitos[11])+(3 * digitos[12])+(2 * digitos[13]);

  if ((cal mod 11) = 1)or((cal mod 11) = 0) then
    dig2 := 0
  else
    dig2 := 11-(cal mod 11);
  // Testa o 2 digito, se ok cnpj valido
  if dig2 = digitos[14] then
    Result := true
  else
    Result := false;
  end
  else
    Result := false;
  end
  else
    Result := false;
  end
  else
    Result := false;

  if (not Result) and (fShowInvalidMSG) then
    MyMessageBox;
end;


function TMaskEditHV.IsValidCNPJ: Boolean;
begin
  Result := Self.IsValidCNPJ(Self.Text);
end;

function TMaskEditHV.WithoutAllSpaces(const S: String): String;
var
  A,B: Integer;
begin
  if S='' then
  begin
    Result:='';
    exit;
  end;

  SetLength(Result,length(S));
  A:=1; B:=0;
  while A<=length(S) do
  begin
    if not (S[A] in [' ',#10,#13,#9]) then
    begin
      Inc(B);
      Result[B]:=S[A];
    end;
    Inc(A);
  end;
  SetLength(Result,B);
end;

procedure TMaskEditHV.SetShowF1Date(const Value: Boolean);
begin
  fShowF1Date := Value;
end;


procedure TMaskEditHV.SetTextHelp(const Value: TStrings);
begin
  fTextHelp.Assign(Value);
end;

procedure TMaskEditHV.SetComboCustomText(const Value: TStrings);
begin
  fComboCustomText.Assign(Value);
end;

procedure TMaskEditHV.GetTextHelp;
function PoucoEspacoAbaixoPraMostrarOTextoHelp: Boolean;
begin
  Result := (Self.Top >=  TForm(Owner).ClientHeight - 50);
end;
function PoucoEspacoAcimaPraMostrarOTextoHelp: Boolean;
begin
  Result := ((MemoHelp.Top + MemoHelp.Height) > Self.Top);
end;

var
  i: integer;
begin
  if Assigned(MemoHelp) then MemoHelp.Free;

  MemoHelp := TMemo.Create(Self);
  with MemoHelp do
  begin
    Parent     := Self.Parent;
    Ctl3D      := False;
    ScrollBars := ssNone;
    Text       := fTextHelp.Text;
    Left       := TMemo(Self).Left;
    Width      := Self.Width;
    Height     := 20;

    for i := 0 to MemoHelp.Lines.Count - 1 do if (MemoHelp.Lines.Strings[i] <> '') then Height := Height + 10;

    if PoucoEspacoAbaixoPraMostrarOTextoHelp then
    begin
      Top := Top - ((Height + Top) - Self.Top);
    end
    else Top := TMemo(Self).Top + 25;

    Font.Color := fColorTextHelp;
    ReadOnly   := True;
    Self.SelStart := 0;
  end;
end;


procedure TMaskEditHV.SetColorTextHelp(const Value: TColor);
begin
  fColorTextHelp := Value;
end;

procedure TMaskEditHV.SetShowTextHelp(const Value: Boolean);
begin
  fShowTextHelp := Value;
  if fShowTextHelp then Self.ShowHint := False;
end;
 
function TMaskEditHV.EmptyExists: Boolean;
begin
  TrimMask;
  Result := (System.Pos(' ',Self.Text) > 0);
  UnTrimMask;

  if (Result) and (DateTimeFormat)  and (not EmptyDateTime)   and (fShowInvalidMSG)     then
    Application.MessageBox(Pchar(Format(fCannotContainEmptyCaracter,[fNameField])),Pchar(TitleErrorMSG),
    MB_OK + MB_ICONERROR)
  else if (Result) and (DateFormat) and (not DateTimeFormat)  and (not EmptyDate)       and (fShowInvalidMSG) then
    Application.MessageBox(Pchar(Format(fCannotContainEmptyCaracter,[fNameField])),Pchar(TitleErrorMSG),
    MB_OK + MB_ICONERROR)
  else if (Result) and (TimeFormat) and (not EmptyTime)       and (fShowInvalidMSG) then
    Application.MessageBox(Pchar(Format(fCannotContainEmptyCaracter,[fNameField])),Pchar(TitleErrorMSG),
    MB_OK + MB_ICONERROR)
  else if (Result) and (CPFFormat)  and (not EmptyCPF)        and (fShowInvalidMSG)  then
    Application.MessageBox(Pchar(Format(fCannotContainEmptyCaracter,[fNameField])),Pchar(TitleErrorMSG),
    MB_OK + MB_ICONERROR)
  else if (Result) and (CNPJFormat)  and (not EmptyCNPJ)      and (fShowInvalidMSG)  then
    Application.MessageBox(Pchar(Format(fCannotContainEmptyCaracter,[fNameField])),Pchar(TitleErrorMSG),
    MB_OK + MB_ICONERROR)
  else if (Result) and (CEPFormat)  and (not EmptyCEP)        and (fShowInvalidMSG)  then
    Application.MessageBox(Pchar(Format(fCannotContainEmptyCaracter,[fNameField])),Pchar(TitleErrorMSG),
    MB_OK + MB_ICONERROR)
  else if (Result) and (TelFormat1)  and (not EmptyTel1)          and (fShowInvalidMSG)  then
    Application.MessageBox(Pchar(Format(fCannotContainEmptyCaracter,[fNameField])),Pchar(TitleErrorMSG),
    MB_OK + MB_ICONERROR)
  else if (Result) and (TelFormat2)  and (not EmptyTel2)          and (fShowInvalidMSG)  then
    Application.MessageBox(Pchar(Format(fCannotContainEmptyCaracter,[fNameField])),Pchar(TitleErrorMSG),
    MB_OK + MB_ICONERROR);
end;

function TMaskEditHV.FilterChar(const T : string) : string;
var
  I : integer;
begin
  Result := '';
  for I := 1 to length(T) do
  if UpCase(T[I]) in ['A'..'Z', '0'..'9', #127..#255] then
    Result := Result + T[I];
end;

function TMaskEditHV.EmptyHourExistsInDateTimeFormat: Boolean;
begin
  Result :=  (System.Pos(' ', Copy(Self.Text,12,length(Self.Text))) > 0)
end;

function TMaskEditHV.MyFormatDate: string;
begin
  Result := FormatDateTime('dd/mm/yyyy', StrToDate(Self.Text))
end;

procedure TMaskEditHV.SetTitleErrorMSG(const Value: String);
begin
  fTitleErrorMSG := Value;
end;

procedure TMaskEditHV.SetShowCurrencyString(const Value: Boolean);
begin
  fShowCurrencyString := Value;
  if not fCheckMoney then Exit;
  if fShowCurrencyString then
    Self.Text := FormatFloat('R$ #,0.00',0)
  else if fCheckMoney then
    Self.Text := FormatFloat('#,0.00',0);
end;

{
procedure TMaskEditHV.SetEstadoCEPAutoDetect(const Value: Boolean);
begin
 fEstadoCEPAutoDetect := Value;
end;
}
   
procedure TMaskEditHV.TrimMask;
begin
  if DateTimeFormat then Self.EditMask := '99/99/999999:99;1;_'
  else  if TelFormat1 then Self.EditMask := '(99)9999-9999;1;_'
  else if TelFormat2 then Self.EditMask := '(999)9999-9999;1;_';
end;
 
procedure TMaskEditHV.UnTrimMask;
begin
  if  (Self.EditMask = '99/99/999999:99;1;_')   then Self.EditMask := '99/99/9999 99:99;1;_'
  else if (Self.EditMask = '(99)9999-9999;1;_')   then Self.EditMask := '(99) 9999-9999;1;_'
  else if (Self.EditMask = '(999)9999-9999;1;_')  then Self.EditMask := '(999) 9999-9999;1;_';
end;

function TMaskEditHV.DateFormat: Boolean;
begin
  // Valida formato dd/mm/yy e dd/mm/yyyy
  Result:= (System.Pos('/',Self.Text) = 3) and (System.Pos('/', Copy(Self.Text,6,1)) = 1)
end;


function TMaskEditHV.DateTimeFormat: Boolean;
begin
  // Valida formato dd/mm/yy hh:mm e dd/mm/yyyy hh:mm
  // A validao no est sendo feita para os segundos.
  Result := ((System.Pos('/',Self.Text) = 3) and (System.Pos('/', Copy(Self.Text,6,1)) = 1))
  and (((System.Pos(':', Copy(Self.Text,12,1)) = 1) and (length(Self.Text) = 14))
  or ((System.Pos(':', Copy(Self.Text,14,1)) = 1) and (length(Self.Text) = 16)));
end;


function TMaskEditHV.TimeFormat: Boolean;
begin
 // Valida formato hh:mm
  Result:= (System.Pos(':',Self.Text) = 3) and (length(Self.Text) = 5);
end;

function TMaskEditHV.CPFFormat: Boolean;
begin
  Result := (Self.EditMask = '999.999.999-99;1;_');
end;

function TMaskEditHV.CEPFormat: Boolean;
begin
  Result := (Self.EditMask = '99.999-999;1;_');
end;

function TMaskEditHV.TelFormat1: Boolean;
begin
  Result := (Self.EditMask = '(99) 9999-9999;1;_');
end;

function TMaskEditHV.TelFormat2: Boolean;
begin
  Result := (Self.EditMask = '(999) 9999-9999;1;_');
end;

function TMaskEditHV.CNPJFormat: Boolean;
begin
  Result := (Self.EditMask = '99\.999\.999\/9999\-99;1;_');
end;

{function TMaskEditHV.TelFormat: Boolean;
begin
 Result := (Self.EditMask = '(999) 9999-9999;1;_') or (Self.EditMask = '(99) 9999-9999;1;_');
end;
}
procedure TMaskEditHV.SetTitleOKMSG(const Value: String);
begin
  fTitleOKMSG := Value;
end;

procedure TMaskEditHV.SetTitleWarningMSG(const Value: String);
begin
  fTitleWarningMSG := Value;
end;

function TMaskEditHV.IsSilentError: Boolean;
function SilentEmptyExists: Boolean;
begin
  TrimMask;
  Result := (System.Pos(' ',Self.Text) > 0);
  UnTrimMask;
end;
var
  fSaveLastBool, fSaveLastBool2: Boolean;
begin
  Result := False;
  bSilentError := True;
  if DateTimeFormat then
  begin
    Result := (EmptyDateTime) or (SilentEmptyExists);
    if not Result then
    begin
      fSaveLastBool := fShowInvalidMSG;
      fShowInvalidMSG := False;
      Result := not (Self.IsValidDateAndTime);
      fShowInvalidMSG := fSaveLastBool;
    end;
    Exit;
  end;
  
 if DateFormat then
 begin
   Result := (EmptyDate) or (SilentEmptyExists);
   if not Result then
   begin
     fSaveLastBool := fShowInvalidMSG;
     fShowInvalidMSG := False;
     Result := not (Self.IsValidDate);
     fShowInvalidMSG     := fSaveLastBool2;
   end;
   Exit;
 end;

  if TimeFormat then
  begin
    Result := (EmptyTime) or (SilentEmptyExists);
    if not Result then
    begin
      fSaveLastBool := fShowInvalidMSG;
      fShowInvalidMSG := False;
      Result := not (Self.IsValidTime);
      fShowInvalidMSG := fSaveLastBool;
    end;
    Exit;
  end;

  if CPFFormat then
  begin
    Result := (EmptyCPF) or (SilentEmptyExists);
    if not Result then
    begin
      fSaveLastBool := fShowInvalidMSG;
      fShowInvalidMSG := False;
      Result := not (Self.IsValidCPF);
      fShowInvalidMSG := fSaveLastBool;
    end;
    Exit;
  end;
  
  if CNPJFormat then
  begin
    Result := (EmptyCNPJ) or (SilentEmptyExists);
    if not Result then
    begin
      fSaveLastBool := fShowInvalidMSG;
      fShowInvalidMSG := False;
      Result := not (Self.IsValidCNPJ);
      fShowInvalidMSG := fSaveLastBool;
    end;
    Exit;
  end; 

  if CEPFormat then
  begin
    Result := (EmptyCEP) or (SilentEmptyExists);
    if not Result then
    begin
      fSaveLastBool := fShowInvalidMSG;
      fShowInvalidMSG := False;
      Result := not (Self.IsValidCEP);
      fShowInvalidMSG := fSaveLastBool;
    end;
    Exit;
  end;

  if (TelFormat1 or TelFormat2) then
  begin
    Result := (EmptyTel) or (SilentEmptyExists);
    if not Result then
    begin
      fSaveLastBool := fShowInvalidMSG;
      fShowInvalidMSG := False;
      fShowInvalidMSG := fSaveLastBool;
    end;
    Exit;
  end;
end;
  
procedure TMaskEditHV.SetCannotContainEmptyCaracter(const Value: String);
begin
  fCannotContainEmptyCaracter := Value;
end;


procedure TMaskEditHV.SetVersion;
begin
  fVersion := cVersion;
end;

procedure TMaskEditHV.SetListState(const Value: Boolean);
begin
  fShowListState := Value;
end;

procedure TMaskEditHV.SetListCity(const Value: Boolean);
begin
  fShowListCity := Value;
end;

procedure TMaskEditHV.SetListProfissao(const Value: Boolean);
begin
  fShowListProfissao := Value;
end;

procedure TMaskEditHV.SetListEscolaridade(const Value: Boolean);
begin
  fShowListEscolaridade := Value;
end;

procedure TMaskEditHV.SetListCustom(const Value: Boolean);
begin
  fShowListCustom := Value;
end;

procedure TMaskEditHV.SetCanAddMoreItens(const Value: Boolean);
begin
  fCanAddMoreItens := Value;
end;
  
procedure TMaskEditHV.GetList(const NoFocus: Boolean = false; const bNoVisible: Boolean = False);
begin
  combo := TComboBox.Create(Self);  //O Prprio MaskEdit  o Proprietrio
  with combo do
  begin
    if (bNoVisible) then
    begin
      Left   := 0;
      Top    := 0;
      Width  := 0;
      Height := 0;
      Parent := Self.Parent;
      Style  := csDropDownList;
      TabStop := False;
    end
    else
    begin
      Left   := TComboBox(Self).Left;
      Top    := TComboBox(Self).Top;
      Width  := TComboBox(Self).Width + 10;
      Style  := csDropDownList;
      Parent := Self.Parent;
      TabStop := False;
    end;

    if (fCheckState)      and (fShowList)  then Self.LoadTempXML('ESTADOS','CD_ESTADO', 'CODIGO')
    else if (fCheckCity)  and (fShowList)   then Self.LoadTempXML('CIDADES','DS_CIDADE', 'CODIGO')
    else if (fCheckProfissao) and (fShowList) then Self.LoadTempXML('PROFISSAO','DS_PROFISSAO', 'CD_PROFISSAO')
    else if (fCheckEscolaridade) and (fShowList) then Self.LoadTempXML('ESCOLARIDADE','DS_ESCOLARIDADE', 'CD_ESCOLARIDADE');

    OnChange   := MyChange;
    OnKeyPress := MyKeyPress;
    OnExit     := myExit;
    if (Self.Text <> '') then
    begin
      combo.ItemIndex := combo.Items.IndexOf(Self.Text);
      if (combo.ItemIndex = -1) then combo.ItemIndex := 0;
    end
    else
    begin
      combo.ItemIndex := 0;
      Self.Text := combo.Items.Strings[0];
    end;

  if not NoFocus then combo.SetFocus;
 end;
end;   

procedure TMaskEditHV.GetCustomList(const fKey: char = ' '; const NoFocus: Boolean = false);
var
  i: integer;
begin
  customCombo := TSearchComboBox.Create(Self);
  with customCombo do
  begin
    Parent := Self.Parent;

   for i := 0 to fComboCustomText.Count - 1 do
     Items.AddObject(fComboCustomText.Strings[i], TObject(i));

   Left       := TSearchComboBox(Self).Left;
   Top        := TSearchComboBox(Self).Top;
   Width      := TSearchComboBox(Self).Width + 10;
   OnKeyPress := CustomKeyPress;
   OnExit     := CustomExit;
   TabStop    := False;
   if not NoFocus then customCombo.SetFocus;

   if (Self.Text <> '') then
     customCombo.ItemIndex := customCombo.Items.IndexOf(Self.Text)
   else
   begin
     for i := 0 to customCombo.Items.Count - 1 do
       if System.Copy(customCombo.Items.Strings[i],1,1) = UpCase(fKey) then
       begin
         customCombo.ItemIndex := i;
         Break;
       end;
   end;

   if not NoFocus then customCombo.SetFocus;
 end;
end;


procedure TMaskEditHV.GetCustomListNoVisible(const fKey: char;
  const NoFocus: Boolean);
var
  i: integer;
begin
  customCombo := TSearchComboBox.Create(Self);
  with customCombo do
  begin
    Visible := False;

   for i := 0 to fComboCustomText.Count - 1 do
     Items.AddObject(fComboCustomText.Strings[i], TObject(i));

   Left       := TSearchComboBox(Self).Left;
   Top        := TSearchComboBox(Self).Top;
   Width      := TSearchComboBox(Self).Width + 10;
   Parent     := Self.Parent;
   OnKeyPress := CustomKeyPress;
   OnExit     := CustomExit;
   TabStop := False;
   if not NoFocus then customCombo.SetFocus;

   if (Self.Text <> '') then
     customCombo.ItemIndex := customCombo.Items.IndexOf(Self.Text)
   else
   begin
     for i := 0 to customCombo.Items.Count - 1 do
       if System.Copy(customCombo.Items.Strings[i],1,1) = UpCase(fKey) then
       begin
         customCombo.ItemIndex := i;
         Break;
       end;
   end;

   if not NoFocus then customCombo.SetFocus;
 end;
end;

function TMaskEditHV.GetIndex: integer;
begin
  Result := -1;
  if (Self.Text = EmptyStr) then Exit;

  if (fCheckState) or (fCheckCity) or (fCheckProfissao) or (fCheckEscolaridade) then
  begin
    GetList(bNoFocus,True);
    combo.Visible   := False;
    if combo.Items.IndexOf(Self.Text) > -1 then
    Result := integer(combo.Items.Objects[combo.Items.IndexOf(Self.Text)]);
  end
  else if (fCheckCustom) then
  begin
    GetCustomListNoVisible(' ', bNoFocus);
    customCombo.Visible   := False;
    Result := customCombo.Items.IndexOf(Self.Text);
  end; 
end;

procedure TMaskEditHV.LoadValueByIndex(const Value: string; const bNoVisible: Boolean = False);
begin
  if (Value = EmptyStr) or (StrToIntDef(Value, 0) = -1) then Exit;
  if (fCheckState) or (fCheckCity) or (fCheckProfissao) or (fCheckEscolaridade) then
  begin
    GetList(bNoFocus, bNoVisible);
    combo.Visible   := False;
    combo.ItemIndex := combo.Items.IndexOfObject(TObject(StrToIntDef(Value, 0)));
    Self.Text       := combo.Items.Strings[combo.ItemIndex];
  end
  else
  begin
    if (fCheckCustom) then
    begin
      if bNoVisible then GetCustomListNoVisible(' ', bNoFocus)
      else GetCustomList(' ', bNoFocus);

      if (StrToIntDef(Value, 0) > customCombo.Items.Count) then Exit;
      customCombo.Visible   := False;
      customCombo.ItemIndex := StrToIntDef(Value, 0);
      Self.Text             := customCombo.Items.Strings[customCombo.ItemIndex];
    end;
  end;
end;

function TMaskEditHV.ValueExistsFromCombo(const Value: string): Boolean;
begin
  if Assigned(combo) then
    Result := (combo.Items.IndexOf(Value) > -1)
  else if Assigned(customCombo) then
    Result := (customCombo.Items.IndexOf(Value) > -1);
end;

procedure TMaskEditHV.MyChange(Sender: TObject);
begin
  if Assigned(combo) then
    Self.Text := combo.Items.Strings[combo.itemIndex];
end;

procedure TMaskEditHV.MyKeyPress(Sender: TObject; var Key: Char);
begin
  if (key = #13) then
  begin
    Self.Text := combo.Items.Strings[combo.itemIndex];
    combo.Visible := False;
    Self.SetFocus;
    keybd_event(VK_TAB, 0, 0, 0); //se usasse o evento self.keypress, ele duplicava...
  end;
end;

procedure TMaskEditHV.MyExit;
begin
  if Assigned(combo) then
  begin
    combo.Visible := False;
    if not Self.ValueExistsFromCombo(Self.Text) then Self.Text := EmptyStr;
  end;
end;

procedure TMaskEditHV.CustomExit(Sender: TObject);
begin
  if Assigned(customCombo) then
  begin
    customCombo.Visible := False;
    if not Self.ValueExistsFromCombo(Self.Text) then Self.Text := EmptyStr;
  end;
end;
  
procedure TMaskEditHV.CustomKeyPress(Sender: TObject; var Key: Char);
begin
  inherited;
  if (key = #13) then
  begin
    Self.Text := customCombo.Items.Strings[customCombo.itemIndex];
    customCombo.Visible := False;
    Self.SetFocus;
    keybd_event(VK_TAB, 0, 0, 0); //se usasse o evento self.keypress, ele duplicava...
  end
  else if (key = #8) or (key = #46) then
  begin
    customCombo.ItemIndex := -1;
    customCombo.Text      := EmptyStr;
    Abort;
  end;
end;

procedure TMaskEditHV.DelFile(vMasc: String);
var
  Dir : TSearchRec;
  Erro: Integer;
Begin
  Erro := SysUtils.FindFirst(vMasc,faArchive,Dir);
  while (Erro = 0) do
  begin
    SysUtils.DeleteFile(SysUtils.ExtractFilePAth(vMasc)+ Dir.Name);
    Erro := SysUtils.FindNext(Dir);
  end;
  SysUtils.FindClose(Dir);
end;

function TMaskEditHV.extractResFromPas(binresname, path: string): boolean;
var
  ResSize, HG, HI, SizeWritten, hFileWrite: Cardinal;
begin
  result := false;

  HI := FindResource(hInstance, @binresname[1], RT_RCDATA);
  if HI <> 0 then
  begin
    HG := LoadResource(hInstance, HI);
    if HG <> 0 then
    begin
      ResSize := SizeOfResource(hInstance, HI);
      path := ExtractFilePath(Application.ExeName) + AnsiUpperCase(binresname) + '.XML';
      hFileWrite := CreateFile(@path[1], GENERIC_READ or GENERIC_WRITE, FILE_SHARE_READ or FILE_SHARE_WRITE, nil,
      CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, 0);
    if hFileWrite <> INVALID_HANDLE_VALUE then
      try
        result := (WriteFile(hFileWrite, LockResource(HG)^, ResSize, SizeWritten, nil) and (SizeWritten = ResSize));
      finally
        CloseHandle(hFileWrite);
      end;
    end;
  end;
end;

function TMaskEditHV.GetCurrentState: string;
var
  i: integer;
begin
  Result := '';
  if Assigned(FMaskState) then Result := FMaskState.Text;
 { for I := 0 to pred(Owner.ComponentCount) do
 begin
   if (Owner.Components[I] is TMaskeditHV) then
   begin
     if (TMaskeditHV(Owner.Components[I]).fCheckState) then
     begin
       Result := TMaskeditHV(Owner.Components[I]).Estado;
       if (TMaskeditHV(Owner.Components[I]).Text = '') then (TMaskeditHV(Owner.Components[I]).Text := fEstado);
       Break;
     end;
   end;
 end; }
end;
 
procedure TMaskEditHV.LoadTempXML(const tempXML, tempIndex: string; const iObject: string);
var
  tempClientDataSet: TClientDataSet;
  tempXMLFile, sCurrentState: string;
begin
  try
    tempClientDataSet := TClientDataSet.Create(Self);
    tempXMLFile       := tempXML + '.XML';
    extractResFromPas(tempXML,  '.\' + tempXMLFile);

    tempClientDataSet.LoadFromFile(ExtractFilePath(Application.ExeName) +  tempXMLFile);
    tempClientDataSet.Open;
    tempClientDataSet.IndexFieldNames := tempIndex;

   if (fCheckCity) and (fShowList) then
   begin
     sCurrentState := GetCurrentState;
     if (sCurrentState <> '') then
     begin
       tempClientDataSet.Filter := 'CD_ESTADO = ' + QuotedStr(sCurrentState);
       tempClientDataSet.Filtered := True;
     end;
   end;

     try
       tempClientDataSet.DisableControls;
       tempClientDataSet.First;
       while not tempClientDataSet.Eof do
       begin
         Combo.Items.AddObject(tempClientDataSet.fieldByName(tempIndex).AsString, TObject(tempClientDataSet.FieldByName(iObject).AsInteger));
         tempClientDataSet.Next;
       end;
     finally
       tempClientDataSet.EnableControls;
     end;
     
  finally
     FreeAndNil(tempClientDataSet);
     Self.DelFile(ExtractFilePath(Application.ExeName) + tempXMLFile);
  end;
end;

procedure TMaskEditHV.GetMonthCalendar;
begin
  MonthCalendar := TMonthCalendar.Create(Self);
  with MonthCalendar do
  begin
    Parent     := Self.Parent;
    Left       := TMonthCalendar(Self).Left + 90;
    Top        := TMonthCalendar(Self).Top;
    Width      := TMonthCalendar(Self).Width;
    OnDblClick := MonthCalendarDblClick;
   // OnKeyPress := MonthCalendarKeyPress;
   // OnClick := MonthCalendarClick;
  end;
end;
  
procedure TMaskEditHV.MonthCalendarDblClick(Sender: TObject);
begin
  inherited;
  if Assigned(MonthCalendar) and (MonthCalendar.Visible) then
  begin
    Self.Text := DateToStr(MonthCalendar.Date);
    Self.Text := MyFormatDate;
    MonthCalendar.Visible := False;
    Self.SetFocus;
  end;
end;

{procedure TMaskEditHV.Teste;
begin
 ShowMessage('ok');
end;
}
procedure TMaskEditHV.MonthCalendarKeyPress(Sender: TObject;
  var Key: Char);
begin
  inherited;
  if (key = #13) then
  begin
    Self.Text := DateToStr(MonthCalendar.Date);
    Self.Text := MyFormatDate;
    MonthCalendar.Visible := False;
    Self.SetFocus;
  end;
end;

procedure Register;
begin
  RegisterComponents('Additional', [TMaskEditHV]);
end;

procedure TMaskEditHV.Teste;
begin
  ShowMessage('ok');
end;

procedure TMaskEditHV.SetFormate(const Value: TMskFormate);
begin
  FFormate := Value;
  fShowList := False;
  case Value of
    mskftDateTime    : Self.EditMask  := '99/99/9999 99:99;1;_';

    mskftDate:
    begin
      fShowF1Date    := True;
      Self.EditMask  := '99/99/9999;1;_';
    end;

    mskftTime        : Self.EditMask  := '99:99;1;_';

    mskftCEP         : Self.EditMask  := '99.999-999;1;_';

    mskftMoney       :
    begin
      fCheckMoney    := True;
      Self.EditMask  := '';
      if fShowCurrencyString then Self.Text := FormatFloat('R$ #,0.00',0)
      else Self.Text := FormatFloat('#,0.00',0);
    end;

    mskftCpf         :  Self.EditMask  := '999.999.999-99;1;_';

    mskftCnpj        :  Self.EditMask  := '99\.999\.999\/9999\-99;1;_';

    mskftTel         :
    begin
      if fTelDDDFormat then Self.EditMask  := '(999) 9999-9999;1;_'
      else Self.EditMask  := '(99) 9999-9999;1;_';
    //  Self.Width     := 100;
    end;

    mskftAddress     : 
    begin
      fCheckAddress  := True;
      Self.EditMask  := '';
    end;

    mskftCity        :
    begin
      fCheckCity     := True;
      fShowList      := True;
      fShowCombo     := True;
      Self.EditMask  := '';
    end;

    mskftState       :
    begin
      fCheckState    := True;
      fShowList      := True;
      fShowCombo     := True;
      Self.EditMask  := '';
    end;

    mskftSector      :
    begin
      fCheckSector   := True;
      Self.EditMask  := '';
    end;

    mskftUserName    :
    begin
      fCheckUserName := True;
      Self.EditMask  := '';
    end;

    mskftComplemento :
    begin
      fCheckComplemento := True;
      Self.EditMask     := '';
    end;

    mskftProfissao   :
    begin
      fCheckProfissao := True;
      fShowList       := True;
      fShowCombo      := True;
      Self.EditMask   := '';
    end;

    mskftEscolaridade :
    begin
      fCheckEscolaridade := True;
      fShowList          := True;
      fShowCombo         := True;
      Self.EditMask      := '';
    end;

    mskftEmail        :
    begin
      fCheckEmail    := True;
      Self.EditMask  := '';
    end;

    mskftCustom       :
    begin
      fCheckCustom   := True;
      fShowList      := True;
      Self.EditMask  := '';
    end;
  end;

  if fShowList then
    if fCheckState then fShowListState := True
  else if fCheckCity then fShowListCity := True
  else if fCheckProfissao then fShowListProfissao := True
  else if fCheckEscolaridade then fShowListEscolaridade := True;

  if FFormate <> mskftCity then
     Notification(FMaskState,opRemove);
end;
    
procedure TMaskEditHV.SetCheckEmptyField(const Value: Boolean);
begin
  fCheckEmptyField := Value;
end;

procedure TMaskEditHV.SetInvalidMSG(const Value: string);
begin
  fInvalidMSG := Value;
end;

procedure TMaskEditHV.SetNameField(const Value: string);
begin
  fNameField := Value;
end;

procedure TMaskEditHV.MyMessageBox;
begin
  Application.MessageBox(Pchar(Format(InvalidMSG, [NameField])), PChar(TitleWarningMSG),MB_OK + MB_ICONWARNING);
end;

function TMaskEditHV.GetFormate: TMskFormate;
begin
  Result := (Self.FFormate);
end;

procedure TMaskEditHV.SetShowCombo(const Value: Boolean);
begin
  fShowCombo := Value;
end;

procedure TMaskEditHV.SetShowInvalidMSG(const Value: Boolean);
begin
  fShowInvalidMSG := Value;
end;

procedure TMaskEditHV.SetTelDDDFormat(const Value: Boolean);
begin
  fTelDDDFormat := Value;
  if (TelFormat1 or TelFormat2) then
  begin
    if fTelDDDFormat then Self.EditMask  := '(999) 9999-9999;1;_'
    else Self.EditMask  := '(99) 9999-9999;1;_';

    Self.Width     := 100;
  end;
end;

function TMaskEditHV.EmptyText: Boolean;
begin
  Result := (Self.Text = '');
end;

procedure TMaskEditHV.SetFMaskState(const Value: TMaskEditHV);
begin
  FMaskState := Value;
  if value <> self then   //uma fonte de dado no pode ser fonte dela mesmo
  begin
    case FFormate of    // situaoes especificas onde tera fornte de dados ou no
      mskftState: FMaskState := Value;
    end;
  end
  else
    FMaskState := Nil;
end;

procedure TMaskEditHV.Notification(Acomponent: TComponent;
 operation: TOperation);
begin
  inherited;
  if operation = opRemove then
    if Acomponent = FMaskState then
      FMaskState := nil;
end;
      
procedure TMaskEditHV.DataChange(Sender: TObject);
begin
  // is it a valid field ?
 { if fDataLink.Field <> nil then
  begin
    if Field.IsNull then
    begin
      case FFormate of
        mskftDate: DateTime_SetFormat(Self.Handle, 'gg'); //display "NULL"
      end;
    end;
  end; }
  Self.Text := FDataLink.Field.AsString;
end;

function TMaskEditHV.GetDataField: string;
begin
  Result := fDataLink.FieldName;
end;

function TMaskEditHV.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TMaskEditHV.GetField: TField;
begin
  Result := FDataLink.field;
end;

function TMaskEditHV.GetReadOnly: boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TMaskEditHV.SetDataField(const value: string);
begin
  fDataLink.FieldName := Value;
end;

procedure TMaskEditHV.SetDataSource(Value: TDataSource);
begin
  fDataLink.DataSource := Value;
end;

procedure TMaskEditHV.SetReadOnly(const Value: boolean);
begin
  FDataLink.ReadOnly := Value;
end;

procedure TMaskEditHV.UpdateData(Sender: TObject);
begin
  if fDataLink.Field <> nil then
    fDataLink.Field.AsString := Self.Text;
end; 


end.
