{*******************************************************}
{       Classes dos objetos utilizados no sistema       }
{                                                       }
{         Exemplo de como utilizar.                     }
{         Copyright (c) 2010-2012, Hamden Vogel         }
{                                                       }
{*******************************************************}


unit UnClasseUsuario;


interface
uses Forms, Windows, Classes, SysUtils, DBClient, DB, Grids, StdCtrls, CheckLst,
Dialogs, IniFiles, ComCtrls, CommCtrl, Controls, UnConexao, clipbrd;


type tAliasBDConexao = (bdConexaoXML, bdConexaoDBISAM, bdConexaoFirebird);


type TUsuario = class
private
 FCd_Usuario: string;
 FNome: string;
 FDataNascimento: string;
 FEndereco: string;
 FCidade: string;
 FEstado: string;
 FCEP: string;
 FCargo: string;
 FSetor: string;
 FTelefone: string;
 FSalario: string;
 FMatricula: string;

 FMensagem: string;
 FCodigoMensagem: integer;
 Alterou:   boolean;
 Conexao: TConexao;
 fStringList: TStringList;
 procedure SetCd_Usuario(const Value: string);
 procedure setNome(const Value: string);
 procedure setDataNascimento(const Value: String);
 procedure setEndereco(const Value: string);
 procedure setCidade(const Value: String);
 procedure setEstado(const Value: String);
 procedure setCEP(const Value: String);
 procedure SetCargo(const Value: string);
 procedure setSetor(const Value: String);
 procedure setTelefone(const Value: String);
 procedure SetSalario(const Value: string);
 procedure SetMatricula(const Value: string);
 procedure ObterDataSet(const myDataSet: TDataSet;  var myClientDataSet: TClientDataSet);


 public


 property CodigoMensagem: integer         read FCodigoMensagem;
 property Cd_Usuario: string read FCd_Usuario write SetCd_Usuario;
 property Mensagem: string    read FMensagem;
 property Nome: string read FNome write setNome;
 property DataNascimento: String read FDataNascimento write setDataNascimento;
 property Endereco: string read FEndereco write setEndereco;
 property Cidade: string read FCidade write setCidade;
 property Estado: string read FEstado write setEstado;
 property CEP: string read FCEP write setCEP;
 property Cargo: string read FCargo write SetCargo;
 property Setor: string read FSetor write setSetor;
 property Telefone: string read FTelefone write setTelefone;
 property Salario: string read FSalario write SetSalario;
 property Matricula: string read FMatricula write SetMatricula;



 function Persistir: boolean;
 function Excluir: Boolean;


 procedure CarregaUsuarios(var myClientDataSet: TClientDataSet);
 function AbreTabelaUsuarios: TDataSet;
 function AbrePesquisaPorNomeDoUsuario: TDataSet;
 function SalvarFoto: Boolean;


 function AbreRecordSetUsuario: TDataSet;
 function AbreRecordSetFiltradoPeloNome: TDataSet;
 function AbreRecordSetSomenteNomes: TDataSet;
 function AbreFoto: TDataSet;
 function RemoverFoto: TDataSet;
 procedure CarregaFoto(var myClientDataSet: TClientDataSet);
 procedure ExcluiFoto(var myClientDataSet: TClientDataSet);
 function UsuarioExiste: Boolean;




 function CarregaNomes: TStringList;
 procedure CarregaRegistrosDeUmNome(const NomePesquisa: string);


 constructor   Create(const fbdConexao: tAliasBDConexao = (bdConexaoDBISAM));
 procedure     SetDataBase(const fbdConexao: tAliasBDConexao);
 destructor    Destroy; override;


end;


implementation


uses UnConst, Biblioteca, MaskEditHV;


{ TUsuario }


function TUsuario.AbreTabelaUsuarios: TDataSet;
begin
 Result := Conexao.RetornaDataSet(sqlSelectAllFieldsOrderByNome);
end;


function TUsuario.AbrePesquisaPorNomeDoUsuario: TDataSet;
begin
 Result := Conexao.RetornaDataSet('SELECT CD_USUARIO, NOME, DATANASCIMENTO, ENDERECO, CIDADE, ESTADO, CEP, CARGO, SETOR, ' +
 'TELEFONE, VLR_SALARIO, NR_MATRICULA FROM USUARIOS WHERE NOME = ' + QuotedStr(Nome));
end;




function TUsuario.AbreRecordSetFiltradoPeloNome: TDataSet;
begin
 Result := Conexao.RetornaDataSet(sqlSelectAllFieldsWhereNome + QuotedStr(Nome));
end;


function TUsuario.AbreRecordSetUsuario: TDataSet;
begin
 Result := Conexao.RetornaDataSet(sqlSelectAllFields);
end;


function TUsuario.AbreRecordSetSomenteNomes: TDataSet;
begin
 Result := Conexao.RetornaDataSet(sqlSelectNome);
end;


constructor TUsuario.Create(const fbdConexao: tAliasBDConexao = (bdConexaoDBISAM));
begin
 Conexao := Conexao.GetConexao(tBDConexao(fbdConexao));
end;


procedure TUsuario.SetDataBase(const fbdConexao: tAliasBDConexao);
begin
 if Assigned(Conexao) then
 begin
   Conexao.Free;
   Conexao := nil;
 end;
 Conexao := Conexao.GetConexao(tBDConexao(fbdConexao));
end;






destructor TUsuario.Destroy;
begin
 if Assigned(fStringList) then FreeAndNil(fStringList);
 inherited;
end;






procedure TUsuario.ObterDataSet(const myDataSet: TDataSet;
 var myClientDataSet: TClientDataSet);
begin
 Conexao.ObterDataSet(myDataSet, myClientDataSet);
end;


function TUsuario.Persistir: boolean;
var
 Query: String;


begin
 Result := False;

 setNome(FNome);  //obrigatrio
 if not CodigoMensagem = CodMensagemNormal then exit;


 setDataNascimento(FDataNascimento);  //obrigatrio
 if not CodigoMensagem = CodMensagemNormal then exit;


 setEndereco(FEndereco);
 if not CodigoMensagem = CodMensagemNormal then exit;


 setEstado(FEstado);   //obrigatrio
 if not CodigoMensagem = CodMensagemNormal then exit;


 setCidade(FCidade);   //obrigatrio
 if not CodigoMensagem = CodMensagemNormal then exit;


 setCEP(FCEP);
 if not CodigoMensagem = CodMensagemNormal then exit;


 SetCargo(FCargo);
 if not CodigoMensagem = CodMensagemNormal then exit;


 setSetor(FSetor);
 if not CodigoMensagem = CodMensagemNormal then exit;


 setTelefone(FTelefone);
 if not CodigoMensagem = CodMensagemNormal then exit;


 SetSalario(FSalario);
 if not CodigoMensagem = CodMensagemNormal then exit;


 SetMatricula(FMatricula);
 if not CodigoMensagem = CodMensagemNormal then exit;



 Query := 'SELECT * FROM USUARIOS WHERE NOME = ' + QuotedStr(Nome);


 if Conexao.RecordCount(Query) > 0 then //atualiza
 begin
   Query := 'UPDATE USUARIOS SET ENDERECO = ' + QuotedStr(Endereco) + ', DATANASCIMENTO = ' + QuotedStr(Conexao.FormatDate(DataNascimento)) +
   ', CIDADE = ' + QuotedStr(Cidade) + ', ESTADO = ' + QuotedStr(Estado) +
   ', CEP = ' + QuotedStr(CEP) + ', CARGO = ' + QuotedStr(Cargo) + ', SETOR = ' + QuotedStr(Setor) +
   ', TELEFONE = ' + QuotedStr(Telefone) + ', VLR_SALARIO = ' + (Conexao.FormatFloat(Salario)) +
   ', NR_MATRICULA = ' + (Conexao.FormatInteger(Matricula)) +
   ' WHERE NOME = ' + QuotedStr(Nome);
   Conexao.Executar(Query);
 end
 else
 begin  //insere
   Query := 'INSERT INTO USUARIOS (CD_USUARIO, NOME, DATANASCIMENTO, ENDERECO, CIDADE, ESTADO, CEP, CARGO, SETOR, ' +
            ' TELEFONE, VLR_SALARIO, NR_MATRICULA) ' +
            'VALUES (' + Conexao.FormatInteger(Conexao.GetNextId('Cd_Usuario','Usuarios')) + ', ' + QuotedStr(Nome) + ', ' +
            QuotedStr(Conexao.FormatDate(DataNascimento)) + ', ' + QuotedStr(Endereco) +
            ', ' + QuotedStr(Cidade) + ', ' + QuotedStr(Estado) + ', ' + QuotedStr(CEP) + ', ' +
            QuotedStr(cargo) + ', ' + QuotedStr(Setor) + ', ' + QuotedStr(Telefone) +  ', ' +
            (Conexao.FormatFloat(Salario)) + ', ' +   (Conexao.FormatInteger(Matricula)) + ' )';
   Conexao.Executar(Query);
 end;


 Alterou := False;
 Result := True;
end;


function TUsuario.Excluir: Boolean;
var
 Query: String;
begin
 Result := False;
 setNome(FNome);


 if not CodigoMensagem = CodMensagemNormal then exit;


 Query := sqlDeleteUsuarioPeloNome + QuotedStr(Nome);
 Conexao.Executar(Query);


 Alterou := False;
 Result := True;
end;




procedure TUsuario.setCEP(const Value: String);
begin
 if Biblioteca.EmptyCEP(Value) then
   FCEP                  := EmptyStr
 else FCEP               := Trim(Value);


 FCodigoMensagem         := CodMensagemNormal;
 FMensagem               := EmptyStr;
 Alterou                 := False;
end;


procedure TUsuario.setCidade(const Value: String); //obrigatrio
begin
 if (Trim(Value) = EmptyStr) or (StrToIntDef(Value, 0) = -1) then
 begin
   FCodigoMensagem         := CodMensagemCampoEmBranco;
   FMensagem               := 'Favor preencher o campo Cidade!';
   FCidade                 := EmptyStr; //necessrio para a funo interna GetValueByIndex do mtodo CarregaRegistrosDeUmNome
 end
 else
 begin
   FCidade                 := Trim(Value);
   FCodigoMensagem         := CodMensagemNormal;
   FMensagem               := EmptyStr;
   Alterou                 := False;
 end;
end;


procedure TUsuario.setDataNascimento(const Value: String); //obrigatrio
var AuxDia, AuxMes, AuxAno, fValue: string;
begin

 fValue := Value;
 fValue := Conexao.GetDateFormat(Value);

 if not (Biblioteca.EmptyDate(fValue)) then
 begin
   if not Biblioteca.IsValidDate(fValue) then
   begin
     FCodigoMensagem       := CodMensagemCampoInvalido;
     FMensagem             := 'Campo Data Nascimento Invlido!';
     FDataNascimento       := '';
   end
   else
   begin
     FCodigoMensagem       := CodMensagemNormal;
     FMensagem             := EmptyStr;
     FDataNascimento       := Conexao.FormatDefaultDate(fValue); //Trim(Value);
   end;
 end
 else
 begin
   FDataNascimento         := fValue;
   FCodigoMensagem         := CodMensagemCampoEmBranco;
   FMensagem               := 'Campo Data Nascimento em Branco!';
   Alterou                 := False;
 end;
end;


procedure TUsuario.setEndereco(const Value: string);
begin
 FEndereco               := Trim(Value);
 FCodigoMensagem         := CodMensagemNormal;
 FMensagem               := EmptyStr;
 Alterou                 := False;
end;


procedure TUsuario.setEstado(const Value: String); //obrigatrio
begin
 if (Trim(Value) = EmptyStr) or (StrToIntDef(Value, 0) = -1) then
 begin
   FCodigoMensagem         := CodMensagemCampoEmBranco;
   FMensagem               := 'Favor preencher o campo UF!';
   FEstado                 := EmptyStr; //necessrio para a funo interna GetValueByIndex do mtodo CarregaRegistrosDeUmNome
 end
 else
 begin
   FEstado                 := Trim(Value);
   FCodigoMensagem         := CodMensagemNormal;
   FMensagem               := EmptyStr;
   Alterou                 := False;
 end;
end;


procedure TUsuario.SetCd_Usuario(const Value: string);
begin
 FCd_Usuario := Value;
end;



procedure TUsuario.setNome(const Value: string); //obrigatrio
begin
 if Trim(Value) = EmptyStr then
 begin
   FCodigoMensagem         := CodMensagemCampoEmBranco;
   FMensagem               := 'Favor preencher o campo Nome!';
 end
 else
 begin
   FNome                   := Trim(Value);
   FCodigoMensagem         := CodMensagemNormal;
   FMensagem               := EmptyStr;
   Alterou                 := False;
 end;
end;


procedure TUsuario.setSetor(const Value: String);
begin
 FSetor                  := Trim(Value);
 FCodigoMensagem         := CodMensagemNormal;
 FMensagem               := EmptyStr;
 Alterou                 := False;
end;


procedure TUsuario.setTelefone(const Value: String);
begin
 if Biblioteca.EmptyFone(Value) then
   FTelefone             := EmptyStr
 else FTelefone          := Trim(Value);


 FCodigoMensagem         := CodMensagemNormal;
 FMensagem               := EmptyStr;
 Alterou                 := False;
end;


procedure TUsuario.SetCargo(const Value: string);
begin
 if (Trim(Value) = EmptyStr) or (StrToIntDef(Value, 0) = -1) then
   FCargo := EmptyStr
 else
   FCargo := trim(Value);


 FCodigoMensagem         := CodMensagemNormal;
 FMensagem               := EmptyStr;
 Alterou                 := False;
end;




procedure TUsuario.SetMatricula(const Value: string);  // obrigatrio
begin
 if (Trim(Value) = EmptyStr) or (Value = '0') then
 begin
   FCodigoMensagem         := CodMensagemCampoEmBranco;
   FMensagem               := 'Favor preencher o campo Matrcula!';
   FMatricula              := '0';
 end
 else
 begin
   FMatricula              := Trim(Value);
   FCodigoMensagem         := CodMensagemNormal;
   FMensagem               := EmptyStr;
   Alterou                 := False;
 end;
end;


procedure TUsuario.SetSalario(const Value: string);
begin
 if (Trim(Value) = EmptyStr) or (Value = '0') then
 begin
   FCodigoMensagem         := CodMensagemCampoEmBranco;
   FMensagem               := 'Favor preencher o campo Salrio!';
   FSalario                := '0';
 end
 else
 begin
   FSalario                := Trim(Value);
   FCodigoMensagem         := CodMensagemNormal;
   FMensagem               := EmptyStr;
   Alterou                 := False;
 end;
end;



function TUsuario.AbreFoto: TDataSet;
begin
 Result := Conexao.RetornaDataSet(sqlSelectAllFieldsWhereNome + QuotedStr(Nome));
 Conexao.FiltroXML := sqlSelectAllFieldsWhereNome + QuotedStr(Nome);
end;


procedure TUsuario.CarregaFoto(var myClientDataSet: TClientDataSet);
begin
 Conexao.ObterDataSet(AbreFoto, myClientDataSet);
end;


function TUsuario.CarregaNomes: TStringList;
var i: integer;
begin
 if not Conexao.BancoInstanciado then
 begin
   Application.MessageBox(PChar(Format(EBancoNaoInstanciado, [FUsuariosFDB])), ETituloErro, MB_OK + MB_ICONERROR);
   Abort;
 end;

 if not Assigned(fStringList) then fStringList := TStringList.Create;

 fStringList.Clear;
 Conexao.ObterDataSet(AbreRecordSetSomenteNomes, fStringList);

 Result := fStringList;
end;


procedure TUsuario.CarregaRegistrosDeUmNome(const NomePesquisa: string);
var
 myDataSet: TDataSet;
begin
 Nome           := NomePesquisa;
 myDataSet      := AbreRecordSetFiltradoPeloNome;

 Cd_Usuario     := myDataSet.FieldByName('Cd_Usuario').AsString;
 Nome           := myDataSet.FieldByName('NOME').AsString;
 DataNascimento := myDataSet.FieldByName('DATANASCIMENTO').AsString;
 Endereco       := myDataSet.FieldByName('ENDERECO').AsString;
 Cidade         := Biblioteca.GetValueByIndex(myDataSet.FieldByName('CIDADE').AsString, mskftCity);
 Estado         := Biblioteca.GetValueByIndex(myDataSet.FieldByName('ESTADO').AsString, mskftState);
 //Cidade         := myDataSet.FieldByName('CIDADE').AsString;
 //Estado         := myDataSet.FieldByName('ESTADO').AsString;
 CEP            := myDataSet.FieldByName('CEP').AsString;
 Cargo          := Biblioteca.GetValueByIndex(myDataSet.FieldByName('CARGO').AsString, mskftProfissao);
 Setor          := myDataSet.FieldByName('SETOR').AsString;
 Telefone       := myDataSet.FieldByName('TELEFONE').AsString;
 Salario        := myDataSet.FieldByName('VLR_SALARIO').AsString;
 Matricula      := myDataSet.FieldByName('NR_MATRICULA').AsString;
end;


procedure TUsuario.ExcluiFoto(var myClientDataSet: TClientDataSet);
begin
 Conexao.ObterDataSet(RemoverFoto, myClientDataSet);
end;


function TUsuario.UsuarioExiste: Boolean;
begin
 Result := (Conexao.RecordCount(AbreRecordSetFiltradoPeloNome) > 0);
end;


function TUsuario.RemoverFoto: TDataSet;
begin
 Result := Conexao.RetornaDataSet(sqlUpdateFotoUsuario + QuotedStr(Nome));
end;


function TUsuario.SalvarFoto: Boolean;
var
 fOpenDialog: TOpenDialog;
 myFile: string;
begin
 setNome(Nome);
 if not CodigoMensagem = CodMensagemNormal then exit;


 fOpenDialog         := TOpenDialog.Create(nil);
 fOpenDialog.Filter  := 'Arquivos BMP (*.bmp)|*.BMP';
 fOpenDialog.Options := [ofOverwritePrompt, ofFileMustExist, ofHideReadOnly];
 if not fOpenDialog.Execute then
 begin
   FreeAndNil(fOpenDialog);
   Exit;
 end;


 try
   myFile := (fOpenDialog.FileName);
 finally
   FreeAndNil(fOpenDialog);
 end;


 if myFile <> EmptyStr then
 begin
  // Self.RemoverFoto;
   Result := conexao.CarregarFoto(myFile, AbreFoto);
 end;
end;


procedure TUsuario.CarregaUsuarios(var myClientDataSet: TClientDataSet);
begin
 Self.ObterDataSet(Self.AbreTabelaUsuarios, myClientDataSet);
end;









end.



***********************************************






{*******************************************************}
{       Classes TConexao e TConexaoXML                  }
{                                                       }
{         Exemplo de como utilizar.                     }
{         Copyright (c) 2010-2012, Hamden Vogel         }
{                                                       }
{*******************************************************}


unit UnConexao;


interface
uses Forms, Classes, Windows, SysUtils, StrUtils, DBClient, IniFiles,Dialogs, DBISAMTb,
DBXpress, SqlExpr, DB, FMTBcd, Provider, UnConst, clipbrd, Graphics;


type tBDConexao = (bdConexaoXML, bdConexaoDBISAM, bdConexaoFirebird);


type EInvalidDataBase = Exception;


type TConexao = class
private
 fFiltroXML: string;
 procedure SetFiltroXML(const Value: string);
 function DefaultDateFormat2(const Value: string): string;
 function DefaultDate(const Value: string): string;
public
 property FiltroXML: string read fFiltroXML write SetFiltroXML;
 function CarregarBanco: Boolean; virtual; abstract;
 function BancoInstanciado: Boolean;
 function BancoExiste: Boolean; virtual; abstract;
 function FecharBanco: Boolean; virtual; abstract;
 function RetornaDataSet(Query: String): TDataSet;
 function CarregarFoto(const MyFile: string; const myDataSet: TDataSet; const MyFieldName: string = 'FOTO'): Boolean;
 procedure Executar(Query: String);
 function RecordCount(Query: String): integer; overload;
 function RecordCount(const myDataSet: TDataSet): integer; overload;
 procedure ObterDataSet(const myDataSet: TDataSet;  var myClientDataSet: TClientDataSet); overload;
 procedure ObterDataSet(const myDataSet: TDataSet;  var myStringList: TStringList); overload;
 function GetConexao(const typeConexao: tBDConexao): tconexao;
 function closeConexao: tconexao;
 function FormatDate(const myDate: string): string;
 function FormatDefaultDate(const MyDate: string): string;
 function FormatFloat(const myFloat: string): string;
 function FormatDefaultFloat(const myFloat: string): string;
 function FormatInteger(const myInteger: string): string;
 function GetDateFormat(const Value: string): string;
 function GetNextId(const fFieldName: string; const fTable: string): string;


end;


//faz uma conexo via XML
type TConexaoXML = class (TConexao)
private
 fCaminhoXMLAtual: string;
 ffBancoXML: string;
 fQuery: string;
 lstSQL: TStringList;


 function ExtractSQLFromSelect(var Str: string): string;
 function ExtractSQLFromInsert(var Str: string): string;
 function ExtractWhereSQLFromSelect(var Str: string): string;
 function ExtractUpdateSQLFromSet(var Str: string): string;
 function ExtractSELECTFromWhere(var Str: string): string;
 function ExtractDELETEFromWhere(var Str: string): string;
 function ExtractINSERTFromValues(var Str: string): string;
 function ExtractUPDATEFromWhere(var Str: string): string;
 function IsAfter(const Str: string; const StrIsAfter: string; const StrToFind: string): Boolean;
 function DeleteChar(const Ch: Char; const S: string): string;
 function Before(const Search, Find: string): string;
 function after(const Search, Find: string): string;
 function AfterWHere(str: string): Boolean;
 function ReplaceFirstOcurrence(const StrToFind: string; const StrToReplace: string): string;
 function DeleteFirstOcurrence(const StrToFind: string; const StrToRemove: string): string;
 function ExtractOrderBy(const StrToFind: string): string;
 function ExtractTableName(const stype: string): string;
 function BancoInstanciado: Boolean;
 function PontoEncontradoAntesDoFinalDaString(const myString: string): Boolean;


 procedure InsertClientDataSetTemp;
 procedure AjustSQL;
 procedure finalAjust;


 function AbrirBanco(const fBancoXML: string; const fIndice: string = ''): Boolean;
 procedure SetCaminhoXMLAtual(const Value: string);
 procedure setBancoXML(const Value: string);
 function BancoExiste(const fBancoXML: string): Boolean;
public
 constructor create(const fBancoXML: string = FHXMLUsuario; const fIndice: string = '');
 destructor destroy; override;
 function FecharBanco: Boolean;
 function RetornaDataSet: TClientDataSet;
 function RecordCount(Query: String): integer;
 function FormatDate(const myDate: string): string;
 function FormatFloat(const myFloat: string): string;
 function FormatInteger(const myInteger: string): string;
 function GetNextId(const fFieldName: string; const fTable: string): string;


 function ExecutaSelect(Query: string): TClientDataSet;
 procedure ExecutaUpdate;
 procedure ExecutaInsert;
 procedure ExecutaDelete;


 property BancoXML: string read ffBancoXML write setBancoXML;
 function CarregarBanco(fBancoXML: string): TClientDataSet;
 procedure Executar(Query: String);
 property CaminhoXMLAtual: string read fCaminhoXMLAtual write SetCaminhoXMLAtual;


end;


type TConexaoDBISAM = class (TConexao)
private
 ffDiretorioDBISAM: string;
 function BancoExiste(const fDiretorio: string; const fShowErrorMSG: Boolean = true): Boolean;
 procedure SetDiretorio(const Value: string);
 procedure FecharBanco;
 function BancoInstanciado: Boolean;
public
 constructor create;
 destructor destroy; override;
 property Diretorio: string read ffDiretorioDBISAM write SetDiretorio;
 function CarregarBanco(const cDiretorio: string = 'DBISAM'): Boolean;
 procedure Executar(Query: String);
 function RecordCount(Query: String): integer;
 function RetornaDataSet(Query: String): TDBISAMQuery; overload;
 function FormatDate(const myDate: string): string;
 function FormatFloat(const myFloat: string): string;
 function FormatInteger(const myInteger: string): string;
 function GetNextId(const fFieldName: string; const fTable: string): string;
end;




type TConexaoFirebird = class (TConexao)
private
 //conexo dbExpress
 fVendorLib: string;
 fLibraryName: string;
 fPathDataBase: string;
 procedure SetVendorLib(const Value: string);
 procedure SetLibraryName(const Value: string);
 procedure SetPathDataBase(const Value: string);
 function BancoInstanciado: Boolean;


public
 constructor create(const cDiretorio: string = 'default');
 destructor destroy; override;
 property dbVendorLib: string read fVendorLib write SetVendorLib;
 property dbLibraryName: string read fLibraryName write SetLibraryName;
 property PathDataBase: string read fPathDataBase write SetPathDataBase;
 function CarregarBanco(const cDiretorio: string = 'default'): Boolean;
 function FecharBanco: Boolean;
 procedure Executar(Query: String);
 function RecordCount(Query: String): integer;

 function RetornaDataSet(Query: String): TDataSet; overload;
 function FormatDate(const myDate: string): string;
 function FormatFloat(const myFloat: string): string;
 function FormatInteger(const myInteger: string): string;
 function GetNextId(const fFieldName: string; const fTable: string): string;

end;




implementation


uses udmDBISAM, UdmXML, udmFIREBIRD, MaskEditHV, Biblioteca;








{ TConexao }


procedure TConexao.Executar(Query: String);
begin
 if Self is TConexaoDBISAM then
   TConexaoDBISAM(Self).Executar(Query)
 else if Self is TConexaoXML then
   TConexaoXML(Self).Executar(Query)
 else if Self is TConexaoFirebird then
   TConexaoFirebird(Self).Executar(Query);
end;


function TConexao.GetConexao(const typeConexao: tBDConexao): TConexao;
begin
 if Assigned(Result) then closeConexao;


 case typeConexao of
  bdConexaoDBISAM:
  begin
    Result := TConexaoDBISAM.create;
  end;

  bdConexaoXML:
  begin
    Result := TConexaoXML.create;
  end;

  bdConexaoFirebird:
  begin
    Result := TConexaoFirebird.create;
  end;
 end;
end;




function TConexao.closeConexao: tconexao;
begin
 if Assigned(dmDBISAM) then
 begin
   dmDBISAM.Free;
   dmDBISAM := nil;
 end
 else if Self is TConexaoXML then
 begin
   if Assigned(dmXML) then
   begin
     dmXML.Free;
     dmXML := nil;
   end;
 end
 else if Self is TConexaoFirebird then
 begin
   if Assigned(dmFirebird) then
   begin
     dmFirebird.Free;
     dmFirebird := nil;
   end;
 end
end;


function TConexao.FormatDate(const myDate: string): string;
begin
 if Self is TConexaoDBISAM then
   Result := TConexaoDBISAM(Self).FormatDate(myDate)
 else if Self is TConexaoXML then
   Result := TConexaoXML(Self).FormatDate(myDate)
 else if Self is TConexaoFirebird then
   Result := TConexaoFirebird(Self).FormatDate(myDate)
end;






function TConexao.FormatDefaultDate(const MyDate: string): string;
begin
 if (MyDate = '') then Exit;
 Result := FormatDateTime('dd/mm/yyyy', StrToDate(MyDate));
end;




function TConexao.FormatDefaultFloat(const myFloat: string): string;
begin
 Result := StringReplace(Result, '.', ',', [rfReplaceAll]);
end;




function TConexao.FormatFloat(const myFloat: string): string;
begin
 if Self is TConexaoDBISAM then
   Result := TConexaoDBISAM(Self).FormatFloat(myFloat)
 else if Self is TConexaoXML then
   Result := TConexaoXML(Self).FormatFloat(myFloat)
 else if Self is TConexaoFirebird then
   Result := TConexaoFirebird(Self).FormatFloat(myFloat)
end;

function TConexao.FormatInteger(const myInteger: string): string;
begin
 if Self is TConexaoDBISAM then
   Result := TConexaoDBISAM(Self).FormatInteger(myInteger)
 else if Self is TConexaoXML then
   Result := TConexaoXML(Self).FormatInteger(myInteger)
 else if Self is TConexaoFirebird then
   Result := TConexaoFirebird(Self).FormatInteger(myInteger)
end;


function TConexao.DefaultDateFormat2(const Value: string): string;
var AuxDia, AuxMes, AuxAno: string;
   IntDia, IntMes, IntAno: smallint;
begin
 Result := Value;
 if (Value = '') then Exit;
 if System.Pos('-', Value) = 0 then Exit;

   AuxAno := copy(Value,1,4);
   AuxMes := copy(Value,6,2);
   AuxDia := copy(Value,9,2);

   IntDia := StrToIntDef(AuxDia, 10);
   IntMes := StrToIntDef(AuxMes, 10);
   IntAno := StrToIntDef(AuxAno, 1900);

   Result := SysUtils.FormatFloat('00', IntDia) + '/' + SysUtils.FormatFloat('00', IntMes) + '/' + IntToStr(IntAno);
end;


function TConexao.DefaultDate(const Value: string): string;
var
 fValue: string;
 AuxDia, AuxMes, AuxAno: string;
begin
 fValue := Value;
 if (fValue <> '') then
 begin
   fValue := FormatDateTime('dd/mm/yyyy', StrToDateDef(fValue, StrToDate('10/10/1900')));
   AuxDia := copy(fValue,1,2);
   AuxMes := copy(fValue,4,2);
   AuxAno := copy(fValue,7,4);
   fValue := SysUtils.FormatFloat('00', StrToIntDef(AuxDia, 0)) + '/' +
   SysUtils.FormatFloat('00', StrToIntDef(AuxMes, 0) ) + '/' + AuxAno;
   Result := fValue;
 end;
end;

function TConexao.GetDateFormat(const Value: string): string;
var
 fValue: string;
begin
 fValue := Value;
 fValue := Self.DefaultDateFormat2(fValue);
 fValue := Self.DefaultDate(fValue);
 Result := fValue;
end;

function TConexao.GetNextId(const fFieldName: string; const fTable: string): string;
begin
 if Self is TConexaoDBISAM then
   Result := TConexaoDBISAM(Self).GetNextId(fFieldName, fTable)
 else if Self is TConexaoXML then
   Result := TConexaoXML(Self).GetNextId(fFieldName, fTable)
 else if Self is TConexaoFirebird then
   Result := TConexaoFirebird(Self).GetNextId(fFieldName, fTable)

end;




procedure TConexao.ObterDataSet(const myDataSet: TDataSet;
 var myClientDataSet: TClientDataSet);
var
 fDataSet: TDataSet;
 i, k: integer;
 FName: string;
 const  arFields       : array [0..2] of string       = ('ESTADO', 'CIDADE','CARGO');
 const  arMskFormates  : array [0..2] of TMskFormate  = (mskftState, mskftCity, mskftProfissao);
begin
 if not Assigned(myDataSet) then Exit;

 if not Self.BancoInstanciado then
 begin
   Application.MessageBox(PChar(Format(EBancoNaoInstanciado, [FUsuariosFDB])), ETituloErro, MB_OK + MB_ICONERROR);
   Abort;
 end;


 fDataSet := TClientDataSet(myDataSet);
 fDataSet.First;


 if not myClientDataSet.Active then
   myClientDataSet.CreateDataSet
 else
   myClientDataSet.EmptyDataSet;


 myClientDataSet.DisableControls;
 try
   while not fDataSet.Eof do
   begin
     myClientDataSet.Insert;
     for i := 0 to fDataSet.FieldCount - 1 do
     begin
      // myClientDataSet.Fields[i].Assign(fDataSet.Fields[i]);

       myClientDataSet.FieldDefs.Items[i].DataType := (fDataSet.FieldDefs.Items[i].DataType);
       myClientDataSet.FieldDefs.Items[i].DisplayName := (fDataSet.FieldDefs.Items[i].DisplayName);
       case myClientDataSet.FieldDefs.Items[i].DataType of
         ftString: myClientDataSet.Fields[i].AsString := Self.DefaultDateFormat2(fDataSet.Fields[i].AsString);
         ftDate:
         begin

            myClientDataSet.Fields[i].AsString := Self.DefaultDateFormat2(fDataSet.Fields[i].AsString);
         end;
       else
        myClientDataSet.Fields[i].AsString  := fDataSet.Fields[i].AsString;

     end;


       for k := 0 to High(arFields) do
         if (AnsiUpperCase(myClientDataSet.Fields[i].DisplayName) = arFields[k]) then
           myClientDataSet.Fields[i].AsString := Biblioteca.GetValueByIndex(myClientDataSet.Fields[i].AsString, arMskFormates[k]);



     end;


     myClientDataSet.Post;
     fDataSet.Next;
   end;
 finally
   myClientDataSet.EnableControls;
 end;
end;




procedure TConexao.ObterDataSet(const myDataSet: TDataSet;
 var myStringList: TStringList);
var
 fDataSet: TDataSet;
 i: integer;
begin
 if not Assigned(myDataSet) then Exit;


 fDataSet := TClientDataSet(myDataSet);
 fDataSet.First;


 if not Assigned(myStringList) then myStringList := TStringList.Create;


 while not fDataSet.Eof do
 begin
   myStringList.Add(fDataSet.Fields[0].AsString);
   fDataSet.Next;
 end;
end;






function TConexao.RecordCount(Query: String): integer;
begin
 if Self is TConexaoDBISAM then
   Result := TConexaoDBISAM(Self).RecordCount(Query)
 else if Self is TConexaoXML then
   Result := TConexaoXML(Self).RecordCount(Query)
 else if Self is TConexaoFirebird then
   Result := TConexaoFirebird(Self).RecordCount(Query);
end;




function TConexao.RecordCount(const myDataSet: TDataSet): integer;
begin
 Result := (TDataSet(myDataSet).RecordCount);
end;




function TConexao.RetornaDataSet(Query: String): TDataSet;
begin
 if Self is TConexaoDBISAM then
   Result := TConexaoDBISAM(Self).RetornaDataSet(Query)
 else if Self is TConexaoXML then
   Result := TConexaoXML(Self).ExecutaSelect(Query)
 else if Self is TConexaoFirebird then
   Result := TConexaoFirebird(Self).RetornaDataSet(Query);
end;




function TConexao.CarregarFoto(const MyFile: string; const myDataSet: TDataSet; const MyFieldName: string = 'FOTO'): Boolean;
function GetTableNameFromXMLSyntax(const mySQLXML: string): string;
begin
 if System.Pos(AnsiUpperCase('where'), AnsiUpperCase(mySQLXML)) > 0 then
   Result := trim(Before(after(AnsiUpperCase(mySQLXML), 'FROM'), 'WHERE'));
end;


var
 S: TFileStream;
 fClientDataSet: TClientDataSet;
 auxTableName: string;
 auxFilterFields: string;
begin
 if Self is TConexaoXML then
 begin


   if (Self.FiltroXML = '') then Exit;


   auxTableName := GetTableNameFromXMLSyntax(Self.FiltroXML);
   auxTableName := auxTableName + '.xml';
   fClientDataSet := TConexaoXML(Self).CarregarBanco(auxTableName);
   auxFilterFields := Before(after(AnsiUpperCase(Self.FiltroXML),'WHERE'), 'ORDER');
   fClientDataSet.Filter := auxFilterFields;
   fClientDataSet.Filtered := True;


   try
     fClientDataSet.Close;
     fClientDataSet.Open;


     if not (fClientDataSet.state in dsEditModes) then fClientDataSet.Edit;


     TBlobField(fClientDataSet.FieldByName(MyFieldName)).LoadFromFile(MyFile);
     fClientDataSet.Post;
     fClientDataSet.SaveToFile(TConexaoXML(Self).CaminhoXMLAtual, dfXML);


   except
     on E: Exception do
     begin
       Raise EInvalidDataBase.Create(EErroGeralBancoDeDados + #13#10 + E.Message);
       Result := False;
     end;
   end;
 end
 else
 if Self is TConexaoDBISAM then
 begin
   S := TFileStream.Create(MyFile ,fmOpenRead);
   with (myDataSet) do
   begin
     try
       try
         Edit;
         TBlobField(FieldByName(MyFieldName)).LoadFromStream(S);
         Post;
       except
         on E: Exception do
         begin
           Raise EInvalidDataBase.Create(EErroGeralBancoDeDados + #13#10 + E.Message);
           Result := False;
         end;
       end;
     finally
       S.Free;
     end;
   end;
 end
 else if Self is TConexaoFirebird then
 begin
   fClientDataSet := dmFirebird.ClUsuario;
   try
     fClientDataSet.Close;
     fClientDataSet.Open;
     fClientDataSet.Refresh;
     if not (fClientDataSet.state in dsEditModes) then fClientDataSet.Edit;


     TBlobField(fClientDataSet.FieldByName(MyFieldName)).LoadFromFile(MyFile);
     fClientDataSet.Post;
     fClientDataSet.ApplyUpdates(0);


   except
     on E: Exception do
     begin
       Raise EInvalidDataBase.Create(EErroGeralBancoDeDados + #13#10 + E.Message);
       Result := False;
     end;
   end;
 end;
end;


function TConexao.BancoInstanciado: Boolean;
begin
 if Self is TConexaoDBISAM then
   Result := TConexaoDBISAM(Self).BancoInstanciado
 else if Self is TConexaoXML then
   Result := TConexaoXML(Self).BancoInstanciado
 else if Self is TConexaoFirebird then
   Result := TConexaoFirebird(Self).BancoInstanciado
end;








{ TConexaoXML }


function TConexaoXML.AbrirBanco(const fBancoXML: string; const fIndice: string): Boolean;
var
 fClientDataSet: TClientDataSet;
 CaminhoDefault, Caminho: string;
begin
 Result := False;
 ffBancoXML := fBancoXML;
 fClientDataSet := CarregarBanco(fBancoXML);
 if fClientDataSet = nil then Exit;


 fClientDataSet.Close;
 CaminhoDefault := ExtractFilePath(Application.ExeName) + 'BD\XML\' + AnsiUpperCase(ffBancoXML); //default
 if fCaminhoXMLAtual = '' then fCaminhoXMLAtual := CaminhoDefault;


 Caminho := fCaminhoXMLAtual;


 if SysUtils.FileExists(Caminho) then
 begin


   fClientDataSet.FileName := AnsiUpperCase(Caminho);
     try
       fClientDataSet.Open;
       Result := True;
       if (fIndice <> '') then
         try
           fClientDataSet.IndexFieldNames := fIndice;
         except
           begin
             raise Exception.CreateFmt(EIndiceInvalido,[fIndice]);
             Result := False;
           end;
         end;
     except
       on E: Sysutils.Exception do
       begin
         Application.MessageBox(Pchar(Format(EErroXML, [fBancoXML]) + #13#10 + E.Message), ETituloErroConexao, MB_OK + MB_ICONERROR);
         Result := False;
       end;
     end;
   end;
end;


function TConexaoXML.BancoExiste(const fBancoXML: string): Boolean;
begin
 Result := SysUtils.FileExists(ExtractFilePath(Application.ExeName) + fBancoXML);
end;


constructor TConexaoXML.create(const fBancoXML: string = FHXMLUsuario; const fIndice: string = '');
begin
 inherited Create;
 Self.AbrirBanco(fBancoXML,fIndice);
end;


destructor TConexaoXML.destroy;
begin
 closeConexao;
 inherited;
end;


function TConexaoXML.FecharBanco: boolean;
begin
 Result := BancoExiste(ffBancoXML);
 if not Result then Exit;
 try
   CarregarBanco(ffBancoXML).Close;
 except
   on E: Sysutils.Exception do
     Application.MessageBox(Pchar(Format(EErroAoFecharArquivo, [ffBancoXML]) + #13#10 + E.Message), ETituloErroConexao, MB_OK + MB_ICONERROR);
 end;
end;




function TConexaoXML.CarregarBanco(fBancoXML: string): TClientDataSet;
begin
 Result := nil;
 if not Assigned(dmXML) then
   dmXML  := TdmXML.Create(nil);
 if (AnsiUpperCase(fBancoXML) = AnsiUpperCase(FHXMLUsuario)) then
    Result := dmXML.ClUsuario;
end;


procedure TConexao.SetFiltroXML(const Value: string);
begin
 fFiltroXML := Value;
end;


procedure TConexaoXML.SetCaminhoXMLAtual(const Value: string);
begin
 if not SysUtils.FileExists(Value) then
 begin
   Application.MessageBox(PChar(Format(EErroXML, [Value])), ETituloErroConexao, MB_OK + MB_ICONERROR);
   fCaminhoXMLAtual := '';
 end
 else fCaminhoXMLAtual := Value;
end;


procedure TConexaoXML.setBancoXML(const Value: string);
begin
 ffBancoXML := Value;
end;


function TConexaoXML.RetornaDataSet: TClientDataSet;
begin
 Result := CarregarBanco(ffBancoXML);
end;


function TConexaoXML.RecordCount(Query: String): integer;
var
 auxFilterFields: string;
 auxTableName: string;
 fClientDataSet: TClientDataSet;


begin
 auxFilterFields := '';
 Result          := 0;
 Self.fQuery := Query;
 auxFilterFields := Before(after(AnsiUpperCase(Query),'WHERE'), 'ORDER');


 auxTableName := trim(ExtractTableName('select'));
 auxTableName := auxTableName + '.xml';


 if not Self.AbrirBanco(auxTableName) then exit;


 fClientDataSet := CarregarBanco(auxTableName);


 fClientDataSet.Close;
 fClientDataSet.Open;


 fClientDataSet.Filter := auxFilterFields;
 try
   fClientDataSet.Filtered := True;
   Result := fClientDataSet.RecordCount;
 except
   fClientDataSet.Filter   := EmptyStr;
   fClientDataSet.Filtered := False;
   Result := 0;
 end;
end;


function TConexaoXML.after(const Search, Find: string): string;
{Retorna uma cadeia de caracteres aps a parte da string selecionada}
var
 index: byte;
begin
 index := Pos(Find, Search);
 if index = 0 then
 begin
   Result := '';
 end
 else
 begin
   Result := Copy(Search, index + Length(Find), 255);
 end;
end;


function TConexaoXML.AfterWHere(str: string): Boolean;
begin
 Result := lstSQL.IndexOf(UpperCase(str)) > lstSQL.IndexOf(UpperCase('where'));
end;


procedure TConexaoXML.AjustSQL;
var
 auxStr: string;
function GetNext: string;
begin
 if not dmXML.ClFormata.Eof then
 begin
   dmXML.ClFormata.Next;
   Result := dmXML.ClFormataDS_TEXTO.AsString;
 end;
end;
function GetNext2: string;
begin
 Result := '';
 while not dmXML.ClFormata.Eof do
 begin
   Result := Result + ' ' + dmXML.ClFormataDS_TEXTO.AsString;
   if  (AnsiEndsText(chr(39),dmXML.ClFormataDS_TEXTO.AsString)) then break;
   dmXML.ClFormata.Next;
  // ClFormata.Prior; // onde estava antes ...
 end;
end;

begin
 dmXML.ClFormata.Open;
 dmXML.ClFormata.First;
 while not dmXML.ClFormata.Eof do
 begin
   if (AnsiStartsText(chr(39), dmXML.ClFormataDS_TEXTO.AsString)) and not (AnsiEndsText(chr(39),dmXML.ClFormataDS_TEXTO.AsString)) then
   begin
     auxStr := GetNext2;
     dmXML.ClFormata.Edit;
     dmXML.ClFormataDS_TEXTO.AsString := trim(auxStr);
     dmXML.ClFormata.Post;
   end;

   dmXML.ClFormata.Next;
 end;
end;


function TConexaoXML.Before(const Search, Find: string): string;
{Retorna uma cadeia de caracteres antecedentes a uma parte da string selecionada}
const
 BlackSpace = [#33..#126];
var
 index: byte;
begin
 index:=Pos(Find, Search);
 if index = 0 then
   Result:=Search
 else
   Result:=Copy(Search, 1, index - 1);
end;


function TConexaoXML.DeleteChar(const Ch: Char; const S: string): string;
var
 Posicao: integer;
begin
 Result := S;
 Posicao := Pos(Ch, Result);
 while Posicao > 0 do
 begin
   Delete(Result, Posicao, 1);
   Posicao := Pos(Ch, Result);
 end;
end;


function TConexaoXML.DeleteFirstOcurrence(const StrToFind,
 StrToRemove: string): string;
var
 i: integer;
 auxStr: string;
begin
 i := System.Pos(StrToRemove, StrToFind);
 if i = 0 then Exit;
 auxStr := TRIM(StrToFind);
 Delete(auxStr, 1, I+2);
 Result := auxStr;
end;


function TConexaoXML.ExtractDELETEFromWhere(var Str: string): string;
var
 i: integer;
 NomeCopiado: string;
begin
 Str := Trim(Str);
 Result := Str;
 Str := Str + #32;
 i := 1;
 lstSQL.Add('WHERE');
 while i > 0 do
 begin


   NomeCopiado := trim(Copy(Str, 1, i - 1));
   NomeCopiado := DeleteChar(',', NomeCopiado);


   if IsAfter(AnsiUpperCase(NomeCopiado), 'WHERE', fQuery) and (NomeCopiado <> '=') then
   begin
     if AnsiUpperCase(NomeCopiado) <> 'HERE' then
       lstSQL.Add(NomeCopiado);


   end;
     Delete(Str, 1, i);
     i := Pos(#32, Str);
 end;
end;


function TConexaoXML.ExtractINSERTFromValues(var Str: string): string;
var
 i: integer;
 NomeCopiado: string;
begin
 Str := Trim(Str);
 Result := Str;
 Str := Str + #32;
 i := 1;

 lstSQL.Add('VALUES');
 while i > 0 do
 begin
   NomeCopiado := trim(Copy(Str, 1, i - 1));
   NomeCopiado := DeleteChar(',', NomeCopiado);

   System.Delete(NomeCopiado, System.Pos('(',NomeCopiado),1);
   System.Delete(NomeCopiado, System.Pos(')',NomeCopiado),1);

 //   if IsAfter(AnsiUpperCase(NomeCopiado), 'VALUES', edtInsert.Text) then
 //   begin
    if AnsiUpperCase(NomeCopiado) <> 'ALUES' then
       lstSQL.Add(NomeCopiado);

     Delete(Str, 1, i);
     i := Pos(#32, Str);
 end;
end;


function TConexaoXML.ExtractOrderBy(const StrToFind: string): string;
begin
 if System.Pos('BY', AnsiUpperCase(StrToFind)) = 0 then Result := ''
 else
  Result := System.Copy(StrToFind, System.Pos('BY', AnsiUpperCase(StrToFind))
 + length('BY') ,length(StrToFind));
end;


function TConexaoXML.ExtractSELECTFromWhere(var Str: string): string;
var
 i: integer;
 NomeCopiado: string;
 auxWhere: string;
begin
 Str := Trim(Str);


 Str := Str + #32;
 lstSQL.Add('WHERE');


 auxWhere := after(AnsiUpperCase(Str), 'WHERE');


 i := 1;
 while i > 0 do
 begin


   NomeCopiado := trim(Copy(auxWhere, 1, i - 1));
   NomeCopiado := DeleteChar(',', NomeCopiado);
   if AnsiUpperCase(NomeCopiado) = AnsiUpperCase('order') then break;


 if (NomeCopiado <> '=') then
 begin
   if (AnsiUpperCase(NomeCopiado) <> 'HERE') and (NomeCopiado <> '') then
     lstSQL.Add(NomeCopiado);


 end;
     Delete(auxWhere, 1, i);
     i := Pos(#32, auxWhere);
 end;
 Result := Str;
end;


function TConexaoXML.ExtractSQLFromInsert(var Str: string): string;
var
 i: integer;
 NomeCopiado: string;
begin
 Str := Trim(Str);
 Result := Str;
 Str := Str + #32;
 i := 1;
 while i > 0 do
 begin
   if (AnsiStartsText(chr(39), trim(Copy(Str, 1, i - 1)))) and not (AnsiEndsText(chr(39), trim(Copy(Str, 1, i - 1)))) then
     if not AnsiEndsText(',', trim(Copy(Str, 1, i - 1))) then
       NomeCopiado := copy(Str,1,19);

   NomeCopiado := trim(Copy(Str, 1, i - 1));
   NomeCopiado := DeleteChar(',', NomeCopiado);

   if NomeCopiado = UpperCase('values') then break;

   if IsAfter(NomeCopiado, ' (', fQuery)  then
   begin
     System.Delete(NomeCopiado, System.Pos('(',NomeCopiado),1);
     System.Delete(NomeCopiado, System.Pos(')',NomeCopiado),1);
     lstSQL.Add(NomeCopiado);
   end;
     Delete(Str, 1, i);
     i := Pos(#32, Str);
end;

end;


function TConexaoXML.ExtractSQLFromSelect(var Str: string): string;
var
 i: integer;
 NomeCopiado, auxTableName: string;
 fClientDataSet: TClientDataSet;
begin
 Str := Trim(Str);
 Result := Str;
 Str := Str + #32;
 i := 1;


 auxTableName := trim(Before(ExtractTableName('select'), 'ORDER'));
 auxTableName := auxTableName + '.xml';


 if not Self.AbrirBanco(auxTableName) then exit;


 fClientDataSet := CarregarBanco(auxTableName);




 if System.pos('*', fQuery) > 0 then
 begin
   fClientDataSet.GetFieldNames(lstSQL);
 end
 else
 while i > 0 do
 begin
   if (AnsiStartsText(chr(39), trim(Copy(Str, 1, i - 1)))) and not (AnsiEndsText(chr(39), trim(Copy(Str, 1, i - 1)))) then
     if not AnsiEndsText(',', trim(Copy(Str, 1, i - 1))) then
       NomeCopiado := copy(Str,1,19);


   NomeCopiado := trim(Copy(Str, 1, i - 1));
   NomeCopiado := DeleteChar(',', NomeCopiado);


   if NomeCopiado = UpperCase('from') then break;


   if IsAfter(NomeCopiado, 'select', fQuery) and (NomeCopiado <> ',') then
   begin
     if (NomeCopiado <> AnsiUpperCase('elect')) then
       lstSQL.Add(NomeCopiado);
   end;
     Delete(Str, 1, i);
     i := Pos(#32, Str);
 end;
end;


function TConexaoXML.ExtractUPDATEFromWhere(var Str: string): string;
var
 i: integer;
 NomeCopiado: string;
begin
 Str := Trim(Str);
 Result := Str;
 Str := Str + #32;
 i := 1;
 lstSQL.Add('WHERE');
 while i > 0 do
 begin
   NomeCopiado := trim(Copy(Str, 1, i - 1));
   NomeCopiado := DeleteChar(',', NomeCopiado);


   if IsAfter(AnsiUpperCase(NomeCopiado), 'WHERE', fQuery) and (NomeCopiado <> '=') then
   begin
     if AnsiUpperCase(NomeCopiado) <> 'HERE' then
       lstSQL.Add(NomeCopiado);
   end;
     Delete(Str, 1, i);
     i := Pos(#32, Str);
 end;
end;


function TConexaoXML.ExtractUpdateSQLFromSet(var Str: string): string;
var
 i: integer;
 NomeCopiado: string;
begin
 Str := Trim(Str);
 Result := Str;
 Str := Str + #32;
 i := 1;


 while i > 0 do
 begin
   if (AnsiStartsText(chr(39), trim(Copy(Str, 1, i - 1)))) and not (AnsiEndsText(chr(39), trim(Copy(Str, 1, i - 1)))) then
     if not AnsiEndsText(',', trim(Copy(Str, 1, i - 1))) then
       NomeCopiado := copy(Str,1,19);


   NomeCopiado := trim(Copy(Str, 1, i - 1));
   NomeCopiado := DeleteChar(',', NomeCopiado);


   if NomeCopiado = UpperCase('where') then break;


   if IsAfter(NomeCopiado, 'set', fQuery) and (NomeCopiado <> '=') then
   begin
     lstSQL.Add(NomeCopiado);
   end;
     Delete(Str, 1, i);
     i := Pos(#32, Str);
 end;
end;


function TConexaoXML.ExtractWhereSQLFromSelect(var Str: string): string;
var
 i: integer;
 NomeCopiado: string;
begin
 Str := Trim(Str);
 Result := Str;
 Str := Str + #32;
 i := 1;
 lstSQL.Add('WHERE');
 while i > 0 do
 begin
   if (AnsiStartsText(chr(39), trim(Copy(Str, 1, i - 1)))) and not (AnsiEndsText(chr(39), trim(Copy(Str, 1, i - 1))))
    then
    if not AnsiEndsText(',', trim(Copy(Str, 1, i - 1))) then
    begin


    end;


   NomeCopiado := trim(Copy(Str, 1, i - 1));
   NomeCopiado := DeleteChar(',', NomeCopiado);


   if IsAfter(AnsiUpperCase(NomeCopiado), 'WHERE', fQuery) and (NomeCopiado <> '=') then
   begin
     if AnsiUpperCase(NomeCopiado) <> 'HERE' then
       lstSQL.Add(NomeCopiado);
   end;
     Delete(Str, 1, i);
     i := Pos(#32, Str);
 end;
end;


procedure TConexaoXML.finalAjust;
begin
 dmXML.ClFormata.First;
 while not dmXML.ClFormata.Eof do
 begin
   if (AnsiStartsText(chr(39), dmXML.ClFormataDS_TEXTO.AsString)) and not (AnsiEndsText(chr(39),dmXML.ClFormataDS_TEXTO.AsString)) then
   repeat
     dmXML.ClFormata.Delete;
   until (AnsiStartsText(chr(39), dmXML.ClFormataDS_TEXTO.AsString)) and (AnsiEndsText(chr(39),dmXML.ClFormataDS_TEXTO.AsString));

   dmXML.ClFormata.Next;
 end;

 lstSQL.Clear;
 dmXML.ClFormata.First;
 while not dmXML.ClFormata.Eof do
 begin
   lstSQL.Add(dmXML.ClFormataDS_TEXTO.AsString);
   dmXML.ClFormata.Next;
 end;
end;


procedure TConexaoXML.InsertClientDataSetTemp;
var
 i: integer;
begin
 dmXML.ClFormata.Open;
 for i := 0 to lstSQL.Count - 1 do
 begin
   dmXML.ClFormata.Append;
   dmXML.ClFormataDS_TEXTO.AsString := lstSQL.Strings[i];
   dmXML.ClFormata.Post;
 end;

 AjustSQL;
end;


function TConexaoXML.IsAfter(const Str, StrIsAfter,
 StrToFind: string): Boolean;
var
 i, k: integer;
begin
 i := pos(UpperCase(StrIsAfter), UpperCase(StrToFind));
 K := pos(UpperCase(str), UpperCase(StrToFind));
 Result := (k > i);
end;


function TConexaoXML.ReplaceFirstOcurrence(const StrToFind,
 StrToReplace: string): string;
begin
 Result := StuffString(StrToFind,1, length(StrToReplace), StrToReplace);
end;


function TConexaoXML.ExtractTableName(const stype: string): string;
begin
 if (stype = 'select') or (stype = 'delete')  then
 begin
   if  System.Pos(AnsiUpperCase('where'), AnsiUpperCase(fQuery)) > 0 then
     Result :=  Before(after(AnsiUpperCase(fQuery), 'FROM'), 'WHERE')
   else  Result :=  (after(AnsiUpperCase(fQuery), 'FROM'))
  end
 else if stype = 'update' then
 begin
   if  System.Pos(AnsiUpperCase('update'), AnsiUpperCase(fQuery)) > 0 then
     Result := Before(after(AnsiUpperCase(fQuery), 'UPDATE'), 'SET')
 end
 else if stype = 'insert' then
 begin
   if  System.Pos(AnsiUpperCase('into'), AnsiUpperCase(fQuery)) > 0 then
     Result := Before(after(AnsiUpperCase(fQuery), 'INTO'), '(')
 end;
end;


procedure TConexaoXML.Executar(Query: String);
var
 auxStr: string;
function GetFirstWord(const str: string): string;
begin
 Result := AnsiUpperCase(Before(str, ' '));
end;
begin
 fQuery := Query;


 if GetFirstWord(Query) = 'UPDATE' then
 begin
  if not Assigned(lstSQL) then
   lstSQL := TStringList.Create;
   lstSQL.Clear;
   dmXML.ClFormata.Open;
   dmXML.ClFormata.EmptyDataSet;
   auxStr := Query;
   ExtractUpdateSQLFromSet(auxStr);
   ExtractUPDATEFromWhere(auxStr);
   InsertClientDataSetTemp;
   finalAjust;
   ExecutaUpdate;


 end //end Update SQL
 else if GetFirstWord(Query) = 'INSERT' then
 begin
  if not Assigned(lstSQL) then
   lstSQL := TStringList.Create;
   lstSQL.Clear;
   dmXML.ClFormata.Open;
   dmXML.ClFormata.EmptyDataSet;
   auxStr := fQuery;
   ExtractSQLFromInsert(auxStr);
   ExtractINSERTFromValues(auxStr);
   InsertClientDataSetTemp;
   finalAjust;
   ExecutaInsert;
 end
 else if GetFirstWord(Query) = 'DELETE' then
 begin
   if not Assigned(lstSQL) then
   lstSQL := TStringList.Create;
   lstSQL.Clear;
   dmXML.ClFormata.Open;
   dmXML.ClFormata.EmptyDataSet;
   auxStr := fQuery;
   ExtractDELETEFromWhere(auxStr);
   InsertClientDataSetTemp;
   finalAjust;
   ExecutaDelete;
 end;


end;


procedure TConexaoXML.ExecutaDelete;
var
 auxFilterFields, auxTableName: string;
 fClientDataSet: TClientDataSet;
begin


 auxFilterFields := after(AnsiUpperCase(fQuery),'WHERE');
 auxTableName    := trim(ExtractTableName('delete'));
 auxTableName    := auxTableName + '.xml';


 if not Self.AbrirBanco(auxTableName) then exit;


 fClientDataSet := CarregarBanco(auxTableName);


 fClientDataSet.First;
 if (System.Pos('DELETE', AnsiUpperCase(fQuery)) > 0)  and (auxFilterFields = '') then
 begin
   while not fClientDataSet.Eof do fClientDataSet.Delete;
   fClientDataSet.SaveToFile(fCaminhoXMLAtual, dfXML);
 end;


 if (auxFilterFields = '') then Exit;


 fClientDataSet.Filter := auxFilterFields;
 try
   fClientDataSet.Filtered := True;
 except
   fClientDataSet.Filter   := EmptyStr;
   fClientDataSet.Filtered := False;
   Exit;
 end;


 if fClientDataSet.Filtered then
   if fClientDataSet.RecordCount > 0 then //encontrou o registro
   begin
     fClientDataSet.First;
     while not fClientDataSet.Eof do fClientDataSet.Delete;


     fClientDataSet.Filtered := False;
     fClientDataSet.SaveToFile(fCaminhoXMLAtual, dfXML);
   end;
end;


procedure TConexaoXML.ExecutaInsert;
var
 i: integer;
 auxString: string;
 auxFilterFields, auxLocateValues: string;
 auxTableName: string;
 fClientDataSet: TClientDataSet;
 XMLValue: string;


function AfterVALUES(str: string): Boolean;
begin
 Result := lstSQL.IndexOf(UpperCase(str)) > lstSQL.IndexOf(UpperCase('values'));
end;


begin
 auxString := '';
 auxFilterFields := '';
 auxLocateValues := '';


 auxTableName := trim(ExtractTableName('insert'));
 auxTableName := auxTableName + '.xml';


 if not Self.AbrirBanco(auxTableName) then exit;


 fClientDataSet := CarregarBanco(auxTableName);


 for i := 0 to lstSQL.Count - 1 do
   if (lstSQL.IndexOf(AnsiUpperCase(' ')) > 0)  then
     lstSQL.Delete(i);


 dmXML.ClFormata.First;
 repeat
   dmXML.ClFormata.Delete
 until AnsiUpperCase(dmXML.ClFormataDS_TEXTO.AsString) = 'VALUES';


 for i := 0 to 1 do dmXML.ClFormata.Delete; //para apagar o registro "Value" e registros em branco


 dmXML.ClFormata.First;
 fClientDataSet.Insert;


 for i := 0 to lstSQL.Count - 1 do
 begin
   auxString := trim(lstSQL.Strings[i]);
   if (fClientDataSet.FieldDefs.IndexOf(auxString) >= 0)
   and not AfterVALUES(auxString) and (auxString <> '')
   then
   begin
     XMLValue := StringReplace(dmXML.ClFormataDS_TEXTO.AsString, chr(39),'',[rfReplaceAll]);

     if (System.Pos(UpperCase('VLR_'), UpperCase(auxString)) > 0) then
     begin
       if PontoEncontradoAntesDoFinalDaString(XMLValue) then
          fClientDataSet.FieldByName(auxString).AsFloat := StrToFloatDef( (StringReplace(XMLValue, '.', ',', [rfReplaceAll])), 0);

     end
     else
       fClientDataSet.FieldByName(auxString).AsString := XMLValue;

     dmXML.ClFormata.Next;
   end;
 end;


 fClientDataSet.Post;
 fClientDataSet.SaveToFile(fCaminhoXMLAtual, dfXML);
end;


function TConexaoXML.ExecutaSelect(Query: string): TClientDataSet;
var
 i, iTotalFields: integer;
 auxString: string;
 auxFilterFields, auxLocateValues: string;
 auxTableName: string;
 auxOrderString: string;
 fClientDataSet: TClientDataSet;




begin
 auxString := '';
 auxFilterFields := '';
 auxLocateValues := '';
 Result := nil;
 fQuery := Query;




 if not Assigned(lstSQL) then
   lstSQL := TStringList.Create;
 lstSQL.Clear;
 dmXML.ClFormata.Open;
 dmXML.ClFormata.EmptyDataSet;
 fQuery := Query;
 ExtractSQLFromSelect(fQuery);
 ExtractSELECTFromWhere(fQuery);
 InsertClientDataSetTemp;
 finalAjust;




 auxFilterFields := BEFORE(after(AnsiUpperCase(fQuery),'WHERE'), 'ORDER');


 auxTableName :=  trim(Before(AnsiUpperCase(ExtractTableName('select')), 'ORDER'));
 auxTableName := auxTableName + '.xml';


 if not Self.AbrirBanco(auxTableName) then exit;


 fClientDataSet := CarregarBanco(auxTableName);
 fClientDataSet.Close;
 fClientDataSet.Open;


 fClientDataSet.Filter := auxFilterFields;
 try
   fClientDataSet.Filtered := True;
 except
   fClientDataSet.Filter   := EmptyStr;
   fClientDataSet.Filtered := False;
 end;


 auxOrderString := ExtractOrderBy(fQuery);
 if (auxOrderString <> '') then
   fClientDataSet.IndexFieldNames := auxOrderString;


 //criar retorno


 if dmXML.ClientDataSetResult.Active then
   dmXML.ClientDataSetResult.EmptyDataSet;
   dmXML.ClientDataSetResult.FieldDefs.Clear;


 iTotalFields := 0;
 for i := 0 to lstSQL.Count - 1 do
 begin
   if (AnsiUpperCase(trim(lstSQL.Strings[i])) = AnsiUpperCase('where'))
   or (AnsiUpperCase(trim(lstSQL.Strings[i])) = AnsiUpperCase('order')) then Break;
 //  if not AfterWHere(trim(lstSQL.Strings[i]))   then
   begin
     inc(iTotalFields);
     with dmXML.ClientDataSetResult do
     begin
       with FieldDefs do
       begin
         with AddFieldDef do
         begin
           Name := trim(lstSQL.Strings[i]);
           if (UpperCase(Name) = 'FOTO') then
             DataType := ftBlob
           else
             if (System.Pos(UpperCase('DATA'), UpperCase(Name)) > 0) then
               DataType := ftDate
           else
             if (System.Pos(UpperCase('VLR_'), UpperCase(Name)) > 0) then
               DataType := ftFloat
           else
             if (System.Pos(UpperCase('NR_'), UpperCase(Name)) > 0) then
               DataType := ftInteger
           else
           begin
             DataType := ftString;
             Size := 200;
           end;
         end;
       end;
     end;
   end;
 end;


 if dmXML.ClientDataSetResult.Active then dmXML.ClientDataSetResult.Close;
 dmXML.ClientDataSetResult.CreateDataSet;


 if not fClientDataSet.Active then Exit;
 if (fClientDataSet.RecordCount = 0) and (auxFilterFields <> '') then
 begin
   dmXML.ClientDataSetResult.Active := False;
   dmXML.ClientDataSetResult.Active := True;
   Result := dmXML.ClientDataSetResult;
   Exit;
 end
 else if (fClientDataSet.RecordCount = 0) then
 begin
   dmXML.ClientDataSetResult.Active := False;
   dmXML.ClientDataSetResult.Active := True;
   Result := dmXML.ClientDataSetResult;
   Exit;
 end;


 fClientDataSet.First;

 while not fClientDataSet.Eof do
 begin
   dmXML.ClientDataSetResult.Insert;

   for i := 0 to (iTotalFields - 1) do
   begin
     dmXML.ClientDataSetResult.fieldByName(trim(lstSQL.Strings[i])).AsString :=  fClientDataSet.fieldByName(trim(lstSQL.Strings[i])).AsString;

   end;

   dmXML.ClientDataSetResult.Post;


   fClientDataSet.Next;
 end;


 dmXML.ClientDataSetResult.First;
 fClientDataSet.IndexFieldNames := '';


 if (fClientDataSet.Filtered) then
   if (fClientDataSet.Active) then
   begin
     fClientDataSet.Filter := '';
     fClientDataSet.Filtered := False;
   end;


 Result := dmXML.ClientDataSetResult;
end;


procedure TConexaoXML.ExecutaUpdate;
function GetWhereText: string;
var
 i: integer;
begin
 Result := '';
 i := System.pos('WHERE', AnsiUpperCase(fQuery));
 if i > 0 then
   Result := System.copy(fQuery, i+5, length(fQuery));
end;


var
 i: integer;
 auxString: string;
 auxFilterFields, auxLocateValues: string;
 auxTableName: string;
 fClientDataSet: TClientDataSet;
 XMLValue: string;
begin
 auxString := '';
 auxFilterFields := '';
 auxLocateValues := '';


 //auxFilterFields := after(AnsiUpperCase(fQuery),'WHERE');
 auxFilterFields := GetWhereText;


 auxTableName := trim(ExtractTableName('update'));
 auxTableName := auxTableName + '.xml';


 if not Self.AbrirBanco(auxTableName) then exit;
 fClientDataSet := CarregarBanco(auxTableName);

 fClientDataSet.Filter := auxFilterFields;
 try
   fClientDataSet.Filtered := True;
 except
   fClientDataSet.Filter   := EmptyStr;
   fClientDataSet.Filtered := False;
   Exit;
 end;


  fClientDataSet.First;


 while not fClientDataSet.Eof do
 begin
   for i := 0 to lstSQL.Count - 1 do
   begin
     auxString := trim(lstSQL.Strings[i]);
     if (fClientDataSet.FieldDefs.IndexOf(auxString) >= 0)
     and not AfterWHere(auxString)
     then
     begin
       if (i <= lstSQL.Count) then
       begin
         XMLValue := StringReplace(trim(lstSQL.Strings[i+1]), chr(39),'',[rfReplaceAll]);
         fClientDataSet.Edit;

           if (System.Pos(UpperCase('VLR_'), UpperCase(auxString)) > 0) then
           begin
             if PontoEncontradoAntesDoFinalDaString(XMLValue) then
             fClientDataSet.FieldByName(auxString).AsFloat := StrToFloatDef( (StringReplace(XMLValue, '.', ',', [rfReplaceAll])), 0);

           end
           else
             fClientDataSet.FieldByName(auxString).AsString := XMLValue;

         fClientDataSet.Post;
         fClientDataSet.SaveToFile(fCaminhoXMLAtual, dfXML);

       end;
     end;
   end;

   fClientDataSet.Next;
 end;

 fClientDataSet.Filtered := False;
 fClientDataSet.Close;
 fClientDataSet.Open;
end;






function TConexaoXML.FormatDate(const myDate: string): string;
var AuxDia, AuxMes, AuxAno: string;
   IntDia, IntMes, IntAno: smallint;
begin
 AuxDia := copy(myDate,1,2);
 AuxMes := copy(myDate,4,2);
 AuxAno := copy(myDate,7,4);


 IntDia := StrToIntDef(AuxDia, 10);
 IntMes := StrToIntDef(AuxMes, 10);
 IntAno := StrToIntDef(AuxAno, 1900);


 Result := IntToStr(IntDia) + '/' + IntToStr(IntMes) + '/' + IntToStr(IntAno);
end;


function TConexaoXML.FormatFloat(const myFloat: string): string;
begin
 Result := StringReplace(myFloat, '.', '', [rfReplaceAll]);
 Result := StringReplace(Result, ',', '.', [rfReplaceAll]);

 //Result := StringReplace(myFloat, '.', '', [rfReplaceAll]);
 //Result :=  '25,60';
end;


function TConexaoXML.BancoInstanciado: Boolean;
begin
 Result := (Assigned(dmXML));
end;

function TConexaoXML.PontoEncontradoAntesDoFinalDaString(
 const myString: string): Boolean;
var
 fIndicePonto: integer;
 tempString: string;
begin
 Result := False;
 tempString := trim(myString);
 fIndicePonto := system.Pos('.',  trim(tempString));
   if fIndicePonto > 0 then
     Result := fIndicePonto < Length(tempString);
end;

function TConexaoXML.FormatInteger(const myInteger: string): string;
begin
 Result := QuotedStr(myInteger);
end;

function TConexaoXML.GetNextId(const fFieldName: string; const fTable: string): string;
var
 fClientDataSet: TClientDataSet;
 ffTable: string;
 sCodigo: string;
begin
 ffTable := fTable + '.xml';
 fClientDataSet := CarregarBanco(ffTable);
 if fClientDataSet = nil then Exit;

 fClientDataSet.LoadFromFile(fCaminhoXMLAtual);

 with fClientDataSet.Aggregates.Add do
 begin
   Expression := 'MAX(' + fFieldName + ')+1';
   AggregateName := 'Max'+ fFieldName;
   Active := True;
 end;

 fClientDataSet.AggregatesActive := true;
 ShowMessage(fFieldName);

 if (fClientDataSet.FieldByName(fFieldName).IsNull) or (fClientDataSet.FieldByName(fFieldName).AsString = EmptyStr) then
 begin
   sCodigo := '1';
   Result := sCodigo;
 end
 else
 begin
   sCodigo := (fClientDataSet.Aggregates.Find('Max'+fFieldName).Value);
   Result := sCodigo;
 end;
end;

{ TConexaoDBISAM }




function TConexaoDBISAM.BancoExiste(const fDiretorio: string;
 const fShowErrorMSG: Boolean): Boolean;
var
 fBancoDeDados: TStringList;
 i: integer;
begin
 Result := (DirectoryExists(fDiretorio)) and (fDiretorio <> '');
 if Result then
 begin
   fBancoDeDados := TStringList.Create;
   try
     with fBancoDeDados do
     begin
       Add('USUARIOS.BLB');
       Add('USUARIOS.DAT');
       Add('USUARIOS.DBK');
       Add('USUARIOS.IBK');
       Add('USUARIOS.IDX');
     end;


     for i := 0 to fBancoDeDados.Count - 1 do
     if not SysUtils.FileExists(IncludeTrailingPathDelimiter(fDiretorio) + fBancoDeDados.Strings[i]) then
     begin
       if fShowErrorMSG then Application.MessageBox(
       Pchar('Erro: Arquivo ' + fBancoDeDados.Strings[i] + ' no encontrado!'),
       'Erro ao Localizar arquivos.',MB_OK + MB_ICONERROR);
       Result := False;
       Break;
     end;
   finally
     fBancoDeDados.Free;
   end;
 end;
end;


function TConexaoDBISAM.CarregarBanco(const cDiretorio: string): Boolean;
var
 fIniFile: TIniFile;
 fFile: TextFile;
 fPathCfgIni: string;
 fDiretorio: string;
 fNotOK: Boolean;
begin
 fNotOK := False;
 fPathCfgIni := ExtractFilePath(Application.ExeName)+ FCFGIni;

 dmDBISAM := TdmDBISAM.Create(nil);
 dmDBISAM.BD.Connected := False;


 if not SysUtils.FileExists(fPathCfgIni) then
 begin
   AssignFile(fFile, fPathCfgIni);
   Rewrite(fFile);
   Append(fFile);
   Writeln(fFile, '[Configuracao]');
   Writeln(fFile, 'Diretorio= ');
   Flush(fFile);  { ensures that the text was actually written to file }
   CloseFile(fFile);
 end
 else
 begin
   fIniFile := TIniFile.Create(fPathCfgIni);
   with fIniFile do
   begin
     try
       dmDBISAM.BD.Directory := ReadString('Configuracao', 'Diretorio', dmDBISAM.BD.Directory);
       dmDBISAM.BD.Connected := Self.BancoExiste(dmDBISAM.BD.Directory);
     finally
       Free;
     end;
   end;
 end;


 if not (dmDBISAM.BD.Connected) then
 begin
   if Self.BancoExiste((ExtractFilePath(Application.ExeName) + 'BD\' + cDiretorio)  , False) then //tenta achar localmente
   begin
     fIniFile := TIniFile.Create(fPathCfgIni);
     with fIniFile do
     begin
       try
         WriteString('Configuracao','Diretorio', (ExtractFilePath(Application.ExeName) + 'BD\' + cDiretorio));
       finally
         Free;
       end;
     end;
   end
   else if not DirectoryExists(dmDBISAM.BD.Directory) or (dmDBISAM.BD.Directory = '') or not (dmDBISAM.BD.Connected) then
   begin
     fIniFile := TIniFile.Create(fPathCfgIni);
     with fIniFile do
     begin
       try
         WriteString('Configuracao','Diretorio', fDiretorio);
       finally
         Free;
       end;
     end;
     end else fNotOK := True;
   end; //end if bd.connected = false then


 if fNotOK then
 begin
   Result := False;
 end
 else
 begin
   if not (dmDBISAM.BD.Connected) then
   begin
     fIniFile := TIniFile.Create(fPathCfgIni);
     with fIniFile do
     begin
       try
         dmDBISAM.BD.Directory := ReadString('Configuracao', 'Diretorio', dmDBISAM.BD.Directory);
         dmDBISAM.BD.Connected := Self.BancoExiste(dmDBISAM.BD.Directory, False);
       finally
         Free;
       end;
     end;
   end;
 end;
 Result := not fNotOK;
end;


constructor TConexaoDBISAM.create;
begin
 if (ffDiretorioDBISAM = '') then
   Self.CarregarBanco
 else
   Self.CarregarBanco(ffDiretorioDBISAM);
 inherited;
end;


destructor TConexaoDBISAM.destroy;
begin
 Self.FecharBanco;
 closeConexao;
 inherited;
end;




procedure TConexaoDBISAM.Executar(Query: String);
begin
 with dmDBISAM.AuxQuery do
 begin
   SQL.Clear;
   SQL.Add(query);
   ExecSQL;
 end;
end;


procedure TConexaoDBISAM.FecharBanco;
begin
 if dmDBISAM.BD.Connected then
 begin
   dmDBISAM.BD.Connected := False;
   dmDBISAM.BD.Directory := EmptyStr;
 end;
end;




function TConexaoDBISAM.RetornaDataSet(Query: String): TDBISAMQuery;
begin
 with dmDBISAM.AuxQuery do
 begin
   SQL.Clear;
   SQL.Add(query);
   Open;
   RequestLive := True;
   Result := dmDBISAM.AuxQuery;
 end;
end;




function TConexaoDBISAM.RecordCount(Query: String): integer;
begin
 Result := TDBISAMQuery(Self.RetornaDataSet(Query)).RecordCount;
end;




procedure TConexaoDBISAM.SetDiretorio(const Value: string);
begin
 ffDiretorioDBISAM := Value;
 // Self.BancoExiste(ffDiretorioDBISAM);
end;












function TConexaoDBISAM.FormatDate(const myDate: string): string;
var AuxDia, AuxMes, AuxAno: string;
   IntDia, IntMes, IntAno: smallint;
begin
 AuxDia := copy(myDate,1,2);
 AuxMes := copy(myDate,4,2);
 AuxAno := copy(myDate,7,4);


 IntDia := StrToIntDef(AuxDia, 10);
 IntMes := StrToIntDef(AuxMes, 10);
 IntAno := StrToIntDef(AuxAno, 1900);


 Result := IntToStr(IntAno) + '-' + IntToStr(IntMes) + '-' + IntToStr(IntDia);


end;


function TConexaoDBISAM.FormatFloat(const myFloat: string): string;
begin
 Result := StringReplace(myFloat, '.', '', [rfReplaceAll]);
 Result := StringReplace(Result, ',', '.', [rfReplaceAll]);
end;


function TConexaoDBISAM.BancoInstanciado: Boolean;
begin
 Result := (Assigned(dmDBISAM));
end;

function TConexaoDBISAM.FormatInteger(const myInteger: string): string;
begin
 Result := myInteger;
end;

function TConexaoDBISAM.GetNextId(const fFieldName: string; const fTable: string): string;
var
 fId: string;
begin
 fId := TDBISAMQuery(Self.RetornaDataSet(Format(sqlMaxIndexFromFieldNameFormat2, [fFieldName, fTable]))).Fields[0].AsString;
 if (fId = '') then fId := '1';
 Result := fId;
end;

{ TConexaoFirebird }


function TConexaoFirebird.CarregarBanco(const cDiretorio: string): Boolean;
begin
 Result := True;
 if (cDiretorio = EmptyStr) then
 begin
   fVendorLib        := FMyGDSVendorLib;
   fLibraryName      := FMyDBXExpressFirebirdLibraryName;
   fPathDataBase     := FUsuariosFDB;
 end
 else if (cDiretorio = 'default') then
 begin
   fVendorLib        := FMyGDSVendorLib;
   fLibraryName      := ExtractFilePath(Application.ExeName) + IncludeTrailingPathDelimiter('BD\FIREBIRD') + FMyDBXExpressFirebirdLibraryName;
   fPathDataBase     := ExtractFilePath(Application.ExeName) + IncludeTrailingPathDelimiter('BD\FIREBIRD') + fPathDataBase;
 end
 else
 begin
   fVendorLib        := cDiretorio + FMyGDSVendorLib;
   fLibraryName      := cDiretorio + FMyDBXExpressFirebirdLibraryName;
   fPathDataBase     := cDiretorio + fPathDataBase;
 end;

 Result := (  (DirectoryExists(fPathDataBase)) and  (FileExists(fLibraryName))  );
 if not Result then Exit;


 dmFirebird := TdmFirebird.Create(nil);

 try
   try
     with dmFirebird.SQLConnection do
     begin
       Connected   := False;
       VendorLib   := dbVendorLib;
       LibraryName := dbLibraryName;
       Params.Values['Database'] := fPathDataBase + FUsuariosFDB;
     end;
    // dmFirebird.SQLTableUsuario.TableName := FUsuariosTableName;
   finally
     dmFirebird.SQLConnection.Connected := True;
    // dmFirebird.SQLTableUsuario.Open;
     dmFirebird.SQLQuery.Open;
   end;
 except
   on E: Exception do
   begin
     Raise EInvalidDataBase.Create(EErroGeralBancoDeDados + #13#10 + E.Message);
     Result := False;
   end;
 end;
end;




constructor TConexaoFirebird.create(const cDiretorio: string = 'default');
begin
 if not Self.CarregarBanco(cDiretorio) then Exit;
 inherited create;
end;


destructor TConexaoFirebird.destroy;
begin
 closeConexao;
 inherited;
end;




procedure TConexaoFirebird.Executar(Query: String);
begin
 if not Self.BancoInstanciado then
 begin
   Application.MessageBox(PChar(Format(EBancoNaoInstanciado, [FUsuariosFDB])), ETituloErro, MB_OK + MB_ICONERROR);
   Abort;
 end;

 with dmFirebird.SQLQuery do
 begin
   SQL.Clear;
   SQL.Add(query);
   ExecSQL(True);
 end;
end;


function TConexaoFirebird.FecharBanco: Boolean;
begin
 if dmFirebird.SQLConnection.Connected then
   dmFirebird.SQLConnection.Connected := False;
end;


function TConexaoFirebird.RecordCount(Query: String): integer;
begin
 Result := TCustomSQLDataSet(Self.RetornaDataSet(Query)).RecordCount;
end;


function TConexaoFirebird.RetornaDataSet(Query: String): TDataSet;
begin
 if not Self.BancoInstanciado then
 begin
   Application.MessageBox(PChar(Format(EBancoNaoInstanciado, [FUsuariosFDB])), ETituloErro, MB_OK + MB_ICONERROR);
   Abort;
 end;

 with dmFirebird.SQLQuery do
 begin
   SQL.Clear;
   SQL.Add(query);
   Open;
 end;
 Result := dmFirebird.SQLQuery;
end;




procedure TConexaoFirebird.SetPathDataBase(const Value: string);
begin
 fPathDataBase := Value;
end;


procedure TConexaoFirebird.SetLibraryName(const Value: string);
begin
 fLibraryName := Value;
end;


procedure TConexaoFirebird.SetVendorLib(const Value: string);
begin
 fVendorLib := Value;
end;




function TConexaoFirebird.FormatDate(const myDate: string): string;
var AuxDia, AuxMes, AuxAno: string;
   IntDia, IntMes, IntAno: smallint;
begin
 AuxDia := copy(myDate,1,2);
 AuxMes := copy(myDate,4,2);
 AuxAno := copy(myDate,7,4);


 IntDia := StrToIntDef(AuxDia, 10);
 IntMes := StrToIntDef(AuxMes, 10);
 IntAno := StrToIntDef(AuxAno, 1900);


 Result := IntToStr(IntAno) + '-' + IntToStr(IntMes) + '-' + IntToStr(IntDia);


end;


function TConexaoFirebird.FormatFloat(const myFloat: string): string;
begin
 Result := StringReplace(myFloat, '.', '', [rfReplaceAll]);
 Result := StringReplace(Result, ',', '.', [rfReplaceAll]);
end;


function TConexaoFirebird.BancoInstanciado: Boolean;
begin
 Result := (Assigned(dmFirebird));
end;

function TConexaoFirebird.FormatInteger(const myInteger: string): string;
begin
 Result := QuotedStr(myInteger);
end;

function TConexaoFirebird.GetNextId(const fFieldName: string; const fTable: string): string;
var
 fId: string;
begin
 fId := TCustomSQLDataSet(Self.RetornaDataSet(Format(sqlMaxIndexFromFieldNameFormat1, [fFieldName, fTable]))).Fields[0].AsString;
 if (fId = '') then fId := '1';
 Result := fId;
end;

end.
